/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file sound_manager.cpp
 * \brief Implementation of the sound_manager class.
 * \author Julien Jorge
 */
#include "audio/sound_manager.hpp"

#include "audio/sdl_music.hpp"
#include "audio/sdl_sound.hpp"
#include "audio/sample.hpp"

#include <claw/assert.hpp>
#include <claw/exception.hpp>
#include <fstream>
#include <vector>

/*----------------------------------------------------------------------------*/
bool bear::audio::sound_manager::s_initialized = false;

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
bear::audio::sound_manager::sound_manager()
  : m_ears_position(0, 0), m_current_music(NULL), m_sound_volume(1),
    m_music_volume(1)
{

} // sound_manager::sound_manager()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
bear::audio::sound_manager::~sound_manager()
{
  clear();
} // sound_manager::~sound_manager()

/*----------------------------------------------------------------------------*/
/**
 * \brief Delete all sounds and musics. If a music is playing, she will be
 *        stopped
 * \post m_sounds.empty == true.
 * \post m_musics.empty == true.
 */
void bear::audio::sound_manager::clear()
{
  stop_all();

  std::map<std::string, sound*>::iterator it_s;
  std::map<std::string, music*>::iterator it_m;

  for (it_s=m_sounds.begin(); it_s!=m_sounds.end(); ++it_s)
    delete it_s->second;

  for (it_m=m_musics.begin(); it_m!=m_musics.end(); ++it_m)
    delete it_m->second;

  m_samples.clear();
  m_sounds.clear();
  m_musics.clear();
} // sound_manager::clear()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add a sound to the cache.
 * \param name The name of the sound.
 * \param file A stream containing the sound.
 * \pre name is not used by another sound.
 */
void bear::audio::sound_manager::load_sound
( const std::string& name, std::istream& file )
{
  CLAW_PRECOND( !sound_exists(name) );

  if (s_initialized)
    m_sounds[name] = new sdl_sound(file, *this);
  else
    m_sounds[name] = new sound(*this);
} // sound_manager::load_sound()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add a music to the cache.
 * \param name The name of the music.
 * \param file A stream containing the music.
 * \pre name is not used by another music.
 */
void bear::audio::sound_manager::load_music
( const std::string& name, std::istream& file )
{
  CLAW_PRECOND( !music_exists(name) );

  if (s_initialized)
    m_musics[name] = new sdl_music(file, *this);
  else
    m_musics[name] = new music();
} // sound_manager::load_music()

/*----------------------------------------------------------------------------*/
/**
 * \brief Start to play a sound.
 * \param name The name of the sound to play.
 */
void bear::audio::sound_manager::play_sound( const std::string& name )
{
  CLAW_PRECOND( sound_exists(name) );

  sample* s = m_sounds[name]->new_sample();
  m_samples[s] = true;

  s->play();
} // sound_manager::play_sound()

/*----------------------------------------------------------------------------*/
/**
 * \brief Start to play the sound, with an effect.
 * \param name The name of the sound to play.
 * \param effect The effect applied to the sound.
 */
void bear::audio::sound_manager::play_sound
( const std::string& name, const sound_effect& effect )
{
  CLAW_PRECOND( sound_exists(name) );

  sample* s = m_sounds[name]->new_sample();
  m_samples[s] = true;

  s->play( effect );
} // sound_manager::play_sound()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get a sound sample.
 * \param name The name of the sound to get.
 * \pre There is an sound called "name".
 * \remark You are responsible of deleting the sample.
 */
bear::audio::sample*
bear::audio::sound_manager::new_sample( const std::string& name )
{
  CLAW_PRECOND( sound_exists(name) );

  sample* result = m_sounds[name]->new_sample();
  m_samples[result] = false;

  return result;
} // sound_manager::new_sample()

/*----------------------------------------------------------------------------*/
/**
 * \brief Play a music.
 * \param name The name of the music to play.
 * \pre There is an music called "name".
 */
void bear::audio::sound_manager::play_music(const std::string& name)
{
  CLAW_PRECOND( music_exists(name) );

  if (m_current_music != NULL)
    m_current_music->stop();

  m_current_music = m_musics[name];
  m_current_music->play();
} // sound_manager::play_music()

/*----------------------------------------------------------------------------*/
/**
 * \brief Stop a music.
 * \param name The name of the music to stop.
 * \pre There is an music called "name".
 */
void bear::audio::sound_manager::stop_music(const std::string& name)
{
  CLAW_PRECOND( music_exists(name) );

  if (m_current_music == m_musics[name])
    m_current_music->stop();
} // sound_manager::stop_music()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the position of the ears.
 * \param position The new position.
 */
void bear::audio::sound_manager::set_ears_position
( const claw::math::coordinate_2d<double>& position )
{
  m_ears_position = position;
} // sound_manager::set_ears_position()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the position of the ears.
 */
const claw::math::coordinate_2d<double>&
bear::audio::sound_manager::get_ears_position() const
{
  return m_ears_position;
} // sound_manager::get_ears_position()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the volume of the sound.
 * \param v The new volume.
 * \pre 0 <= v <= 1
 */
void bear::audio::sound_manager::set_sound_volume( double v )
{
  CLAW_PRECOND( v >= 0 );
  CLAW_PRECOND( v <= 1 );

  m_sound_volume = v;

  std::map<sample*, bool>::iterator it;

  for ( it=m_samples.begin(); it!=m_samples.end(); ++it )
    it->first->set_volume(m_sound_volume);
} // sound_manager::set_sound_volume()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the volume of the music.
 * \param v The new volume.
 * \pre 0 <= v <= 1
 */
void bear::audio::sound_manager::set_music_volume( double v )
{
  CLAW_PRECOND( v >= 0 );
  CLAW_PRECOND( v <= 1 );

  m_music_volume = v;

  if (m_current_music != NULL)
    m_current_music->set_volume(m_music_volume);
} // sound_manager::set_music_volume()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the volume of the sound.
 */
double bear::audio::sound_manager::get_sound_volume() const
{
  return m_sound_volume;
} // sound_manager::get_sound_volume()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the volume of the music.
 */
double bear::audio::sound_manager::get_music_volume() const
{
  return m_music_volume;
} // sound_manager::get_music_volume()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if a sound is in the cache.
 */
bool bear::audio::sound_manager::sound_exists(const std::string& name) const
{
  return m_sounds.find(name) != m_sounds.end();
} // sound_manager::sound_exists()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if a music is in the cache.
 */
bool bear::audio::sound_manager::music_exists(const std::string& name) const
{
  return m_musics.find(name) != m_musics.end();
} // sound_manager::music_exists()

/*----------------------------------------------------------------------------*/
/**
 * \brief Inform the manager that a sample is finished. If the sample is managed
 *        by the manager, it will be deleted.
 */
void bear::audio::sound_manager::sample_finished( sample* s )
{
  std::map<sample*, bool>::iterator it = m_samples.find(s);

  if ( it!=m_samples.end() )
    if ( it->second )
      {
        m_samples.erase(it);
        delete s;
      }
} // sound_manager::sample_finished()

/*----------------------------------------------------------------------------*/
/**
 * \brief Inform the manager that a sample is deleted. The sample is removed
 *        from the set of samples.
 */
void bear::audio::sound_manager::sample_deleted( sample* s )
{
  m_samples.erase(s);
} // sound_manager::sample_finished()

/*----------------------------------------------------------------------------*/
/**
 * \brief Stop all musics and sounds.
 */
void bear::audio::sound_manager::stop_all()
{
  // some samples can be removed from m_samples when calling sample::stop().
  // avoir looping on a map that changes.
  std::map<sample*, bool>::iterator it_s;
  std::vector<sample*> s;

  s.reserve( m_samples.size() );

  for ( it_s=m_samples.begin(); it_s!=m_samples.end(); ++it_s )
    s.push_back(it_s->first);

  for (unsigned int i=0; i!=s.size(); ++i)
    s[i]->stop();

  if (m_current_music != NULL)
    m_current_music->stop();
} // sound_manager::stop_all()

/*----------------------------------------------------------------------------*/
/**
 * \brief Initialize the sound system.
 */
void bear::audio::sound_manager::initialize()
{
  s_initialized = sdl_sound::initialize();
} // sound_manager::initialize()

/*----------------------------------------------------------------------------*/
/**
 * \brief Close the sound system.
 */
void bear::audio::sound_manager::release()
{
  sdl_sound::release();
  s_initialized = false;
} // sound_manager::release()
