/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file model.tpp
 * \brief Implementation of the bear::engine::model class.
 * \author Julien Jorge
 */
#include "engine/game.hpp"

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
template<class Base>
bear::engine::model<Base>::model()
  : m_sample(NULL), m_animation_alignment(align_center_on_center)
{

} // model::model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy constructor.
 * \param that The instance to copy from.
 */
template<class Base>
bear::engine::model<Base>::model( const model<Base>& that )
{
  assign( that );
} // model::model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
template<class Base>
bear::engine::model<Base>::~model()
{
  end_action();
} // model::~model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assignment operator.
 * \param that The instance to copy from.
 */
template<class Base>
bear::engine::model<Base>&
bear::engine::model<Base>::operator=( const model<Base>& that )
{
  if ( this != &that )
    assign(that);

  return *this;
} // model::operator=()

/*----------------------------------------------------------------------------*/
/**
 * \brief Go to the next frame of the animation.
 */
template<class Base>
void bear::engine::model<Base>::progress( universe::time_type elapsed_time )
{
  m_animation.next(elapsed_time);
} // model::progress()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the sprite representing the item.
 * \param visuals (out) The sprites of the item, and their positions.
 */
template<class Base>
void
bear::engine::model<Base>::get_visual( std::list<scene_visual>& visuals ) const
{
  visuals.push_front
    ( scene_visual
      ( this->get_top_left() + this->get_gap(), m_animation.get_sprite(),
        this->get_angle(), this->get_z_position() ) );
} // model::get_visual()

/*----------------------------------------------------------------------------*/
/**
 * \brief Indicates if the action is finished.
 */
template<class Base>
bool bear::engine::model<Base>::action_is_finished() const
{
  return m_animation.is_finished();
} // model::action_is_finished()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the current animation of the model.
 */
template<class Base>
const bear::visual::animation&
bear::engine::model<Base>::get_current_animation() const
{
  return m_animation;  
} // model::get_current_animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the flip of the animation.
 */
template<class Base>
void bear::engine::model<Base>::flip(bool b)
{
  m_animation.flip(b);
} // model::flip()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the sprite of the animation.
 */
template<class Base>
void bear::engine::model<Base>::mirror(bool b)
{
  m_animation.mirror(b);
} // model::mirror()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the alpha transparence of the animation.
 */
template<class Base>
void bear::engine::model<Base>::set_alpha_blend(double alpha)
{
  m_animation.set_alpha_blend(alpha);
} // model::set_alpha_blend()

/*----------------------------------------------------------------------------*/
/**
 * \brief End the action.
 */
template<class Base>
void bear::engine::model<Base>::end_action()
{
  if ( m_sample != NULL )
    {
      m_sample->stop();
      delete m_sample;
      m_sample = NULL;
    }
} // model::end_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the alignment of the animation and the bounding box.
 * \param align Tell how we adjust the animation and the bounding box.
 */
template<class Base>
void
bear::engine::model<Base>::set_animation_alignment( animation_alignment align )
{
  m_animation_alignment = align;
} // model::set_animation_alignment()

/*----------------------------------------------------------------------------*/
/**
 * \brief Start a new action.
 * \param name The name of the new action.
 * \pre There is an action called \a name.
 */
template<class Base>
void bear::engine::model<Base>::execute_action( const std::string& name )
{
  end_action();

  const model_action a = get_action(name);

  m_animation = a.get_animation();

  if ( !a.get_sound_name().empty() )
    {
      m_sample = this->get_level_globals().new_sample( a.get_sound_name() );
      m_sample->play();
    }

  compute_gap();
} // model::execute_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the model file.
 * \param file_name The path to the model file.
 */
template<class Base>
void bear::engine::model<Base>::set_model( const std::string& file_name )
{
  assign_model( this->get_level_globals().get_model(file_name) );
} // model::set_model()

/*----------------------------------------------------------------------------*/
/**
 * \brief Compute the gap between the animation and the bounding box.
 */
template<class Base>
void bear::engine::model<Base>::compute_gap()
{
  universe::coordinate_type x;
  universe::coordinate_type y;

  const claw::math::coordinate_2d<int> anim_size( m_animation.get_max_size() );

  switch( m_animation_alignment )
    {
    case align_center_on_bottom:
    case align_center_on_top:
    case align_center_on_center:
      x = (this->get_width() - anim_size.x) / 2;
      break;
    case align_center_on_left:
    case align_top_left_on_top_left:
    case align_bottom_left_on_bottom_left:
      x = 0;
      break;
    case align_center_on_right:
    case align_top_right_on_top_right:
    case align_bottom_right_on_bottom_right:
      x = this->get_width() - anim_size.x;
      break;
    }

  switch( m_animation_alignment )
    {
    case align_center_on_bottom:
    case align_bottom_left_on_bottom_left:
    case align_bottom_right_on_bottom_right:
      y = this->get_height() - anim_size.y;
      break;
    case align_center_on_left:
    case align_center_on_right:
    case align_center_on_center:
      y = (this->get_height() - anim_size.y) / 2;
      break;
    case align_center_on_top:
    case align_top_left_on_top_left:
    case align_top_right_on_top_right:
      y = 0;
      break;
    }

  this->set_gap( (int)x, (int)y );
} // model::compute_gap()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assignment.
 * \param that The instance to copy from.
 */
template<class Base>
void bear::engine::model<Base>::assign( const model<Base>& that )
{
  end_action();

  m_animation = that.m_animation;
  m_animation_alignment = that.m_animation_alignment;
} // model::assign()
