/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file model_loader.tpp
 * \brief Implementation of the bear::engine::model_loader class.
 * \author Julien Jorge
 */
#include "engine/model_loader.hpp"

#include <claw/assert.hpp>

#include "engine/level_globals.hpp"
#include "model_code_value.hpp"
#include "engine/sprite_loader.hpp"

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param f The file from which we load the model.
 * \param glob The level_globals in which we load the resources.
 * \param actor The model that we are building.
 *
 * \a f must live longer than this instance of model_loader.
 */
bear::engine::model_loader::model_loader
( std::istream& f, level_globals& glob, model_actor& actor )
  : m_file(f, true), m_level_globals(glob), m_model(actor)
{

} // model_loader::model_loader()

/*----------------------------------------------------------------------------*/
/**
 * \brief Build the model
 */
void bear::engine::model_loader::run()
{
  unsigned int next_code;

  m_file >> next_code;

  while( next_code != model_code_value::eof )
    next_code = load_action();
} // model_loader::run()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add an action to the model.
 * \param name The name of the action.
 * \param animation The animation to play.
 * \param sound_name The sound to play.
 */
void bear::engine::model_loader::add_action
( const std::string& name, const visual::animation& animation,
  const std::string& sound_name )
{
  if ( !sound_name.empty() )
    m_level_globals.load_sound( sound_name );

  model_action a(animation, sound_name);
  m_model.add_action(name, a);
} // model_loader::add_action()

/*----------------------------------------------------------------------------*/
/**
 * \brief Load an action.
 */
unsigned int bear::engine::model_loader::load_action()
{
  unsigned int next_code;
  std::string name, sound;

  m_file >> name;

  const visual::animation anim =
    sprite_loader::load_animation( m_file, m_level_globals );

  m_file >> sound;

  add_action( name, anim, sound );

  m_file >> next_code;

  return next_code;
} // model_loader::load_action()
