/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file with_decoration.cpp
 * \brief Implementation of the bear::engine::with_decoration class.
 * \author Sebastien Angibaud
 */
#include "engine/with_decoration.hpp"
#include <claw/assert.hpp>

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
bear::engine::with_decoration::with_decoration()
  : m_animation(NULL), m_sprite(NULL)
{

} // with_decoration::with_decoration()

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy constructor.
 * \param that The instance to copy from.
 */
bear::engine::with_decoration::with_decoration( const with_decoration& that )
  : m_animation(NULL), m_sprite(NULL)
{
  assign(that);
} // with_decoration::with_decoration()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
bear::engine::with_decoration::~with_decoration()
{
  clear();
} // with_decoration::~with_decoration()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assignment.
 * \param The instance to copy from.
 */
bear::engine::with_decoration&
bear::engine::with_decoration::operator=( const with_decoration& that )
{
  if ( this != &that )
    {
      clear();
      assign(that);
    }

  return *this;
} // with_decoration::operator=()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the sprite.
 * \param spr The new sprite.
 */
void bear::engine::with_decoration::set_sprite( const visual::sprite& spr )
{
  if ( !has_sprite() )
    m_sprite = new visual::sprite(spr);
  else
    *m_sprite = spr;
} // with_decoration::set_sprite()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the animation.
 * \param anim The new animation.
 */
void
bear::engine::with_decoration::set_animation( const visual::animation& anim )
{
  if ( !has_animation() )
    m_animation = new visual::animation(anim);
  else
    *m_animation = anim;
} // with_decoration::set_animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the sprite.
 * \pre has_sprite() == true.
 */
bear::visual::sprite& bear::engine::with_decoration::get_sprite()
{
  CLAW_PRECOND( has_sprite() );

  return *m_sprite;
} // with_decoration::get_sprite()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the animation.
 * \pre has_animation() == true.
 */
bear::visual::animation& bear::engine::with_decoration::get_animation()
{
  CLAW_PRECOND( has_animation() );

  return *m_animation;
} // with_decoration::get_animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the sprite.
 * \pre has_sprite() == true.
 */
const bear::visual::sprite& bear::engine::with_decoration::get_sprite() const
{
  CLAW_PRECOND( has_sprite() );

  return *m_sprite;
} // with_decoration::get_sprite()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the animation.
 * \pre has_animation() == true
 */
const bear::visual::animation&
bear::engine::with_decoration::get_animation() const
{
  CLAW_PRECOND( has_animation() );

  return *m_animation;
} // with_decoration::get_animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if the sprite is defined.
 */
bool bear::engine::with_decoration::has_sprite() const
{
  return m_sprite != NULL;
} // with_decoration::has_sprite()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if the animation is defined.
 */
bool bear::engine::with_decoration::has_animation() const
{
  return m_animation != NULL;
} // with_decoration::has_animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove the sprite, if any.
 */
void bear::engine::with_decoration::remove_sprite()
{
  if ( has_sprite() )
    {
      delete m_sprite;
      m_sprite = NULL;
    }
} // with_decoration::remove_sprite()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove the animation, if any.
 */
void bear::engine::with_decoration::remove_animation()
{
  if ( has_animation() )
    {
      delete m_animation;
      m_animation = NULL;
    }
} // with_decoration::remove_animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove the decorations.
 */
void bear::engine::with_decoration::clear()
{
  remove_animation();
  remove_sprite();
} // with_decoration::clear()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assign the decoration from an other instance.
 * \param that The instance to copy from.
 * \pre this != &that
 */
void bear::engine::with_decoration::assign( const with_decoration& that )
{
  CLAW_PRECOND( this != &that );

  if ( that.has_animation() )
    set_animation( that.get_animation() );

  if ( that.has_sprite() )
    set_sprite( that.get_sprite() );
} // with_decoration::assign()
