/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file model.hpp
 * \brief This class manage some basic informations on items: actions with
 *        their sounds, animations and bounding boxes.
 * \author Julien Jorge
 */
#ifndef __ENGINE_MODEL_HPP__
#define __ENGINE_MODEL_HPP__

#include "engine/basic_renderable_item.hpp"
#include "engine/model_actor.hpp"

namespace bear
{
  namespace engine
  {
    /**
     * \brief This class manage some basic informations on items: actions with
     *        their sounds, animations and bounding boxes.
     *
     * \b Template \b arguments:
     *  - \a Base the base class of the model.
     *
     * \remark This class assume that animations are oriented to the right by
     *         default.
     *
     * \author Julien Jorge
     */
    template<class Base>
    class model:
      public basic_renderable_item<Base>,
      public model_actor
    {
    public:
      /** \brief The type of the parent class. */
      typedef basic_renderable_item<Base> super;

      /** \brief Tell how we adjust the animation and the bounding box. */
      enum animation_alignment
        {
          align_center_on_bottom,
          align_center_on_left,
          align_center_on_top,
          align_center_on_right,
          align_center_on_center,
          align_top_left_on_top_left,
          align_top_right_on_top_right,
          align_bottom_left_on_bottom_left,
          align_bottom_right_on_bottom_right
        }; // enum animation_alignment

    public:
      model();
      model( const model<Base>& that );
      ~model();

      model<Base>& operator=( const model<Base>& that );

      void progress( universe::time_type elapsed_time );
      void get_visual( std::list<scene_visual>& visuals ) const;

      bool action_is_finished() const;

    protected:
      const visual::animation& get_current_animation() const;

      void flip(bool b);
      void mirror(bool b);
      void set_alpha_blend(double alpha);
      void end_action();

      void set_animation_alignment( animation_alignment align );
      void execute_action( const std::string& name );

      void set_model( const std::string& model_file );
      void compute_gap();

    private:
      void assign( const model<Base>& that );

    private:
      /** \brief The animation of the current action. */
      visual::animation m_animation;

      /** \brief The sample of the current action. */
      audio::sample* m_sample;

      /** \brief Tell how we adjust the animation and the bounding box. */
      animation_alignment m_animation_alignment;

    }; // class model
  } // namespace engine
} // namespace bear

#include "engine/code/model.tpp"

#endif // __ENGINE_MODEL_HPP__
