/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file collision_detection.cpp
 * \brief Implementation of the bear::universe::collision_detection class.
 * \author Julien Jorge
 */
#include "universe/collision_detection.hpp"

#include "universe/collision_info.hpp"

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param dynamic_items The set of dynamic items.
 */
bear::universe::collision_detection::collision_detection
( dynamic_item_set& dynamic_items )
  : m_dynamic_items(dynamic_items)
{
  
} // collision_detection::collision_detection()

/*----------------------------------------------------------------------------*/
/**
 * \brief Check for collisions of an item with all items of the neighboorhood.
 * \param d_item (out) The current dynamic item to consider. 
 * \param can_move_item_state The can_move_item state to consider for each 
 * neighbour
 */
bool bear::universe::collision_detection::detect
( dynamic_item& d_item, bool can_move_item_state )
{
  bool result = false;
  dynamic_item::neighbourhood_iterator_type it;
  
  for ( it = d_item.get_neighbourhood().begin(); 
        it != d_item.get_neighbourhood().end(); ++it )
    if ( (*it)->can_move_items() == can_move_item_state )
      {
        const claw::avl<item_pair>::const_iterator eit = m_corrected.end();

        if ( m_corrected.find( item_pair(d_item.get_item(), *it) ) == eit )
          if ( m_corrected.find( item_pair(*it, d_item.get_item()) ) == eit )
            if ( d_item.get_item()->get_bounding_box().intersects
                 ( (*it)->get_bounding_box() ) )
              {
                const rectangle_type inter =
                  d_item.get_item()->get_bounding_box().intersection
                  ( (*it)->get_bounding_box() );

                if( (inter.width !=0) && (inter.height != 0) )
                  {
                    collision( *d_item.get_item(), **it,
                               d_item.get_initial_state(),
                               m_dynamic_items[*it].get_initial_state() );

                    collision( **it, *d_item.get_item(),
                               m_dynamic_items[*it].get_initial_state(),
                               d_item.get_initial_state() );
            
                    result = true;
                    m_corrected.insert( item_pair(d_item.get_item(), *it) );
                  }
              }
      }
  
  return result;
} // collision_detection::detect()

/*----------------------------------------------------------------------------*/
/**
 * \brief Build the collision info and call physical_item::collision()
 * \param self The item on which we call collision().
 * \param that The other item of the collision.
 * \param old_self The state to consider for the the current item.
 * \param old_that The state to consider for the item "that".
 */
void bear::universe::collision_detection::collision
( physical_item& self, physical_item& that, const physical_item_state& old_self,
  const physical_item_state& old_that ) const
{
  collision_info info( old_self, old_that, self, that );
  self.collision(info);
} // collision_detection::collision()
