/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file physical_item.cpp
 * \brief Implementation of the bear::universe::physical_item class.
 * \author Julien Jorge
 */
#include "universe/physical_item.hpp"

#include <algorithm>

#include "universe/collision_info.hpp"
#include "universe/forced_movement.hpp"
#include "universe/link/base_link.hpp"
#include "universe/world.hpp"

#include <claw/assert.hpp>
#include <claw/functional.hpp>
#include <claw/logger.hpp>

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
bear::universe::physical_item::physical_item()
  : m_forced_movement(NULL), m_owner(NULL)
{

} // physical_item::physical_item()

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy constructor.
 * \param that The item to copy from.
 * \remark Links and forced movement are not copied.
 */
bear::universe::physical_item::physical_item( const physical_item& that )
  : m_forced_movement(NULL), m_owner(NULL)
{

} // physical_item::physical_item()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
bear::universe::physical_item::~physical_item()
{
  // force a call to remove_handle, that will remove the handle from the list
  while ( !m_handles.empty() )
    *m_handles.front() = NULL;

  clear_forced_movement();
  remove_all_links();
} // physical_item::~physical_item() [destructor]

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the forced movement applied to this item, if any.
 * \pre this->has_force_movement()
 */
bear::universe::forced_movement&
bear::universe::physical_item::get_forced_movement()
{
  CLAW_PRECOND( has_forced_movement() );

  return *m_forced_movement;
} // physical_item::get_forced_movement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Apply a forced movement to the item.
 * \param m The movement to apply.
 * \remark If this object already have a movement, il will be released.
 */
void
bear::universe::physical_item::set_forced_movement( const forced_movement& m )
{
  clear_forced_movement();

  if ( is_fixed() )
    claw::logger << claw::log_warning
                 << "physical_item::set_forced_movement(): setting a "
                 << "forced movement but the item is fixed." << claw::lendl;

  set_acceleration( force_type(0, 0) );

  m_forced_movement = m.clone();
  m_forced_movement->set_item(*this);
  m_forced_movement->init();
} // physical_item::set_forced_movement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Check if the item has a forced movement.
 */
bool bear::universe::physical_item::has_forced_movement() const
{
  return m_forced_movement != NULL;
} // physical_item::has_forced_movement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove the forced movement, if any.
 */
void bear::universe::physical_item::clear_forced_movement()
{
  if (m_forced_movement)
    {
      delete m_forced_movement;
      m_forced_movement = NULL;
    }
} // physical_item::clear_forced_movement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Apply the movement of the item.
 * \param elapsed_time Elasped time since the last progress.
 */
void bear::universe::physical_item::move( time_type elapsed_time )
{
  default_move(elapsed_time);
} // physical_item::move()

/*----------------------------------------------------------------------------*/
/**
 * \brief Process a collision.
 * \param info Some informations about the collision.
 */
void bear::universe::physical_item::collision( collision_info& info )
{

} // physical_item::collision()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell this item which world owns him.
 * \param The world that owns this item.
 */
void bear::universe::physical_item::set_owner(world& owner)
{
  m_owner = &owner;
} // physical_item::set_owner()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add a link in this item.
 * \param link The link to add.
 * \pre The caller is bear::universe::base_link.
 * \pre The link is not already in the item's list.
 * \remark The item keeps a reference on the link.
 */
void bear::universe::physical_item::add_link( base_link& link )
{
  CLAW_PRECOND
    ( std::find(m_links.begin(), m_links.end(), &link) == m_links.end() );

  m_links.push_front(&link);
} // physical_item::add_link()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove a link from this items.
 * \param link The link to remove.
 * \pre The caller is bear::universe::base_link.
 * \pre The link is in the item's list.
 */
void bear::universe::physical_item::remove_link( base_link& link )
{
  CLAW_PRECOND
    ( std::find(m_links.begin(), m_links.end(), &link) != m_links.end() );

  m_links.erase( std::find(m_links.begin(), m_links.end(), &link) );
} // physical_item::remove_link()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get an iterator at the begining of the list of links.
 */
bear::universe::physical_item::const_link_iterator
bear::universe::physical_item::links_begin() const
{
  return m_links.begin();
} // physical_item::links_begin()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get an iterator at the end of the list of links.
 */
bear::universe::physical_item::const_link_iterator
bear::universe::physical_item::links_end() const
{
  return m_links.end();
} // physical_item::links_end()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add an handle to the item.
 * \pre (h != NULL)
 *      && (std::find(m_handles.begin(), m_handles.end(), h) == m_handles.end())
 */
void bear::universe::physical_item::add_handle( item_handle* h )
{
  CLAW_PRECOND( h != NULL );
  CLAW_PRECOND( std::find(m_handles.begin(), m_handles.end(), h)
                == m_handles.end() );

  m_handles.push_front( h );
} // physical_item::add_handle()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove an handle to the item.
 * \pre (h != NULL)
 *      && (std::find(m_handles.begin(), m_handles.end(), h) != m_handles.end())
 */
void bear::universe::physical_item::remove_handle( item_handle* h )
{
  CLAW_PRECOND( h != NULL );
  CLAW_PRECOND( std::find(m_handles.begin(), m_handles.end(), h)
                != m_handles.end() );

  m_handles.erase( std::find( m_handles.begin(), m_handles.end(), h ) );
} // physical_item::remove_handle()

/*----------------------------------------------------------------------------*/
/**
 * \brief Return the list of items that depend on \a this.
 * \param items A list that receives the dependent items.
 */
void bear::universe::physical_item::get_dependent_items
( std::list<physical_item*>& items )
{

} // physical_item::get_dependent_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if this item knows which world owns him.
 */
bool bear::universe::physical_item::has_owner() const
{
  return m_owner != NULL;
} // physical_item::has_owner()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the world that owns this item.
 * \pre has_owner() == true
 */
bear::universe::world& bear::universe::physical_item::get_owner() const
{
  CLAW_PRECOND( has_owner() );
  return *m_owner;
} // physical_item::get_owner()

/*----------------------------------------------------------------------------*/
/**
 * \brief Apply the default movement of the item (forced_movement or
 *        acceleration and speed).
 * \param elapsed_time Elasped time since the last progress.
 */
void bear::universe::physical_item::default_move( time_type elapsed_time )
{
  if ( has_forced_movement() )
    m_forced_movement->next_position(elapsed_time);
  else
    update_position(elapsed_time);
} // physical_item::default_move()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision.
 * \param info Some informations about the collision.
 */
bool bear::universe::physical_item::default_collision( collision_info& info )
{
  bool result(false);

  switch( info.get_collision_side() )
    {
    case zone::bottom_zone:
      result = collision_align_bottom
        (info.other_item(), info.get_top_left_on_contact());
      break;
    case zone::top_zone:
      result = collision_align_top
        (info.other_item(), info.get_top_left_on_contact());
      break;
    case zone::middle_left_zone:
      result = collision_align_left
        (info.other_item(), info.get_top_left_on_contact());
      break;
    case zone::middle_right_zone:
      result = collision_align_right
        (info.other_item(), info.get_top_left_on_contact());
      break;
    case zone::middle_zone:
      result = false;
      break;
    default: { CLAW_ASSERT( false, "Invalid collision side." ); }
    }

  return result;
} // physical_item::default_collision()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the left of \a this.
 * \param info Some informations about the collision.
 */
bool bear::universe::physical_item::collision_align_left( collision_info& info )
{
  position_type pos;

  if ( info.get_collision_side() == zone::middle_left_zone )
    pos = info.get_top_left_on_contact();
  else
    pos.set( get_left() - info.other_item().get_width(),
             info.other_item().get_top() );

  return collision_align_left( info.other_item(), pos );
} // physical_item::collision_align_left()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the right of \a this.
 * \param info Some informations about the collision.
 */
bool
bear::universe::physical_item::collision_align_right( collision_info& info )
{
  position_type pos;

  if ( info.get_collision_side() == zone::middle_right_zone )
    pos = info.get_top_left_on_contact();
  else
    pos.set( get_right(), info.other_item().get_top() );

  return collision_align_right( info.other_item(), pos );
} // physical_item::collision_align_right()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the top of \a this.
 * \param info Some informations about the collision.
 */
bool bear::universe::physical_item::collision_align_top( collision_info& info )
{
  position_type pos;

  if ( info.get_collision_side() == zone::top_zone )
    pos = info.get_top_left_on_contact();
  else
    pos.set( info.other_item().get_left(),
             get_top() - info.other_item().get_height() );

  return collision_align_top( info.other_item(), pos );
} // physical_item::collision_align_top()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the bottom of \a this.
 * \param info Some informations about the collision.
 */
bool
bear::universe::physical_item::collision_align_bottom( collision_info& info )
{
  position_type pos;

  if ( info.get_collision_side() == zone::bottom_zone )
    pos = info.get_top_left_on_contact();
  else
    pos.set( info.other_item().get_left(), get_bottom() );

  return collision_align_bottom( info.other_item(), pos );
} // physical_item::collision_align_bottom()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the left of \a this.
 * \param that The other item to align.
 * \param pos The top left position to put the other item at.
 */
bool bear::universe::physical_item::collision_align_left
( physical_item& that, const position_type& pos )
{
  bool result(false);

  if ( collision_align_at(that, pos) )
    {
      result = true;

      that.set_right_contact();
      set_left_contact();

      that.set_speed( speed_type(0, that.get_speed().y) );
      that.set_acceleration( force_type(0, that.get_acceleration().y) );
    }

  return result;
} // physical_item::collision_align_left()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the right of \a this.
 * \param that The other item to align.
 * \param pos The top left position to put the other item at.
 */
bool bear::universe::physical_item::collision_align_right
( physical_item& that, const position_type& pos )
{
  bool result(false);

  if ( collision_align_at(that, pos) )
    {
      result = true;

      that.set_left_contact();
      set_right_contact();

      that.set_speed( speed_type(0, that.get_speed().y) );
      that.set_acceleration( force_type(0, that.get_acceleration().y) );
    }

  return result;
} // physical_item::collision_align_right()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the top of \a this.
 * \param that The other item to align.
 * \param pos The top left position to put the other item at.
 */
bool bear::universe::physical_item::collision_align_top
( physical_item& that, const position_type& pos )
{
  bool result(false);

  if ( collision_align_at(that, pos) )
    {
      result = true;

      that.set_bottom_contact();
      set_top_contact();

      that.set_speed( speed_type(that.get_speed().x, 0) );
      that.set_acceleration( force_type(that.get_acceleration().x, 0) );
    }

  return result;
} // physical_item::collision_align_top()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item of the collision on the bottom of \a this.
 * \param that The other item to align.
 * \param pos The top left position to put the other item at.
 */
bool bear::universe::physical_item::collision_align_bottom
( physical_item& that, const position_type& pos )
{
  bool result(false);

  if ( collision_align_at(that, pos) )
    {
      result = true;

      that.set_top_contact();
      set_bottom_contact();

      that.set_speed( speed_type(that.get_speed().x, 0) );
      that.set_acceleration( force_type(that.get_acceleration().x, 0) );
    }

  return result;
} // physical_item::collision_align_bottom()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align an item at a given position.
 * \param that The other item.
 * \param pos The top left position to put the other item at.
 */
bool bear::universe::physical_item::collision_align_at
( physical_item& that, const position_type& pos ) const
{
  bool result(true);

  if ( that.is_phantom() )
    result = false;
  else
    that.set_top_left( pos );

  return result;
} // physical_item::collision_align_at()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove all the links from this items.
 */
void bear::universe::physical_item::remove_all_links()
{
  // The base_link::unlink() method must call physical_item::remove_link() so
  // we don't have to remove it from here.
  while ( !m_links.empty() )
    {
      base_link* link = m_links.front();
      delete link;
    }
} // physical_item::remove_all_links()

/*----------------------------------------------------------------------------*/
/**
 * \brief Update the position, the speed and the acceleration of the item.
 * \param elapsed_time Elasped time since the last progress.
 */
void bear::universe::physical_item::update_position( time_type elapsed_time )
{
  force_type a( get_acceleration() );
  double f = get_friction() * get_contact_friction();

  if ( has_owner() )
    {
      a += get_owner().get_gravity();
      f *= get_owner().get_average_friction( get_bounding_box() );
    }

  position_type pos( get_top_left() );
  speed_type speed( a * elapsed_time + get_speed() );
  
  speed *= f;
  set_speed( speed );

  pos += speed * elapsed_time;
  set_top_left(pos);

  adjust_cinetic();
} // physical_item::update_position()

/*----------------------------------------------------------------------------*/
/**
 * \brief Calculate the new acceleration of the item.
 */
void bear::universe::physical_item::adjust_cinetic()
{
  speed_type s( get_speed() );

  if ( has_owner() )
    {
      const speed_type eps(get_owner().get_speed_epsilon());

      if ( (s.x < eps.x) && (s.x > -eps.x) )
        s.x = 0;
      if ( (s.y < eps.y) && (s.y > -eps.y) )
        s.y = 0;

      set_speed(s);
    }

  set_acceleration( force_type(0, 0) );
  set_speed( s );
} // physical_item::adjust_cinetic()
