/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file universe/code/world.cpp
 * \brief Implementation of the bear::universe::world class.
 * \author Julien Jorge
 */
#include "universe/world.hpp"

#include "universe/link/base_link.hpp"

/*----------------------------------------------------------------------------*/
const unsigned int bear::universe::world::c_map_compression = 256;

/*----------------------------------------------------------------------------*/
claw::math::rectangle<
bear::universe::world::base_entity_traits::coordinate_type>
bear::universe::world::base_entity_traits::get_bounding_box
( base_entity* const & item ) const
{
  CLAW_PRECOND( item != NULL );

  return item->get_bounding_box();
} // world::base_entity_traits::get_bounding_box()

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param size Size of the world.
 */
bear::universe::world::world( const size_box_type& size )
  : m_static_surfaces( (unsigned int)size.x + 1, (unsigned int)size.y + 1,
                       c_map_compression ),
    m_size(size), m_unit(50), m_gravity(0, 9.81*m_unit), m_default_friction(1),
    m_speed_epsilon(1, 1), m_acceleration_epsilon(0.01, 0.01)
{
  
} // world::world()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
bear::universe::world::~world()
{
  for ( ; !m_friction_rectangle.empty(); m_friction_rectangle.pop_front() )
    delete m_friction_rectangle.front();
} // world::~world()

/*----------------------------------------------------------------------------*/
/**
 * \brief Call the progress() method on entities in the active region, then
 *        apply physic rules.
 * \param regions The active regions.
 * \param elapsed_time Elapsed time since the last call of this method.
 */
void bear::universe::world::progress_entities
( const region_type& regions, time_type elapsed_time )
{
  item_set static_items;
  item_set living_items;
  dynamic_item_set dynamic_items;

  lock();

  // search each item in the active zone and global item
  search_interesting_items(regions, static_items, living_items);

  // call progress for each interesting item
  progress_items(static_items, elapsed_time);
  progress_items(living_items, elapsed_time);

  // inform living_item if they go out the active zone
  active_region_traffic( regions, living_items );

  // create a dynamic_item for each living item
  create_dynamic_items( dynamic_items, static_items, living_items );

  // move the item and applied links
  progress( elapsed_time, dynamic_items );

  // detection of collision
  detect_collision_all( dynamic_items );

  unlock();
} // world::progress_entities()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add a static item in the world.
 * \param who The item to add.
 */
void bear::universe::world::add_static(base_entity* who)
{
  CLAW_PRECOND( who != NULL );
  CLAW_PRECOND( !locked() );

  who->fix();
  who->set_owner(*this);
  m_static_surfaces.insert( who );

  if ( who->is_global() )
    m_global_static_items.insert( who );
} // world::add_static()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the size of the world.
 */
const bear::universe::size_box_type& bear::universe::world::get_size() const
{
  return m_size;
} // world::get_size()

/*----------------------------------------------------------------------------*/
/**
 * \brief Print some statistics.
 */
void bear::universe::world::print_stats() const
{
  unsigned int min, max;
  double avg;

  m_static_surfaces.cells_load(min, max, avg);

  claw::logger << claw::log_verbose << "World's size is " << m_size.x << ", "
               << m_size.y << '\n'
               << "Cells' size is " << c_map_compression << '\n'
               << "The loading is (min, max, avg) (" << min << '\t' << max
               << '\t' << avg << ")\n"
               << m_static_surfaces.empty_cells() << " cells are empty\n"
               << "There are " << m_entities.size() << " entities." 
               << claw::lendl;
} // world::print_stats()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the gravity applied to the items.
 */
const bear::universe::force_type& bear::universe::world::get_gravity() const
{
  return m_gravity;
} // world::get_gravity()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the gravity applied to the items.
 * \param g The value of the gravity applied to the items.
 */
void bear::universe::world::set_gravity( const force_type& g )
{
  m_gravity = g;
} // world::set_gravity();

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the gravity applied to the items, call to_world_unit() on the
 *        given value.
 * \param g The value of the gravity applied to the items.
 */
void bear::universe::world::set_scaled_gravity( const force_type& g )
{
  m_gravity.set( to_world_unit(g.x), to_world_unit(g.y) );
} // world::set_scaled_gravity();

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the current speed of the item.
 */
const bear::universe::speed_type&
bear::universe::world::get_speed_epsilon() const
{
  return m_speed_epsilon;
} // world::get_speed_epsilon()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the current speed of the item.
 */
void bear::universe::world::set_speed_epsilon( const speed_type& speed )
{
  m_speed_epsilon = speed;
} // world::set_speed_epsilon()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the current speed of the item, call to_world_unit() on the given
 *        value.
 */
void bear::universe::world::set_scaled_speed_epsilon( const speed_type& speed )
{
  m_speed_epsilon.set( to_world_unit(speed.x), to_world_unit(speed.y) );
} // world::set_scaled_speed_epsilon()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the unit of the world. \a u units == 1 meter.
 * \param u The new unit.
 */
void bear::universe::world::set_unit( coordinate_type u )
{
  m_unit = u;
} // world::set_unit()

/*----------------------------------------------------------------------------*/
/**
 * \brief Convert meters in the units of the world.
 * \param m The value to convert.
 */
bear::universe::coordinate_type
bear::universe::world::to_world_unit( coordinate_type m ) const
{
  return m * m_unit;
} // world::to_world_unit()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the unit the default friction applied to the items by the
 *        environment.
 * \param f The new friction.
 */
void bear::universe::world::set_default_friction( double f )
{
  m_default_friction = f;
} // world::set_default_friction()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the average friction in an area of the world.
 * \param r The area to search in.
 */
double
bear::universe::world::get_average_friction( const rectangle_type& r ) const
{
  const double r_area( r.area() );
  double result(0);

  if (r_area != 0)
    {
      std::list<friction_rectangle*>::const_iterator it;
      double sum_area(0);

      for ( it=m_friction_rectangle.begin(); it!=m_friction_rectangle.end();
            ++it )
        if ( r.intersects( (*it)->rectangle ) )
          {
            const double area = r.intersection( (*it)->rectangle ).area();
            result += area / r_area * (*it)->friction;
            sum_area += area;
          }

      if ( sum_area < r_area )
        result += (r_area - sum_area) / r_area * m_default_friction;
    }

  return result;
} // world::get_average_friction()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add a region with an environment friction different
 *        from m_default_friction.
 * \param r The region of the world where the friction is different.
 * \param f The friction in this region.
 * \return The friction_rectangle stored by the world. You can change its values
 *         as you need.
 */
bear::universe::friction_rectangle*
bear::universe::world::add_friction_rectangle
( const rectangle_type& r, double f )
{
  m_friction_rectangle.push_front( new friction_rectangle(r, f) );
  return m_friction_rectangle.front();
} // world::add_friction_rectangle()

/*----------------------------------------------------------------------------*/
/**
 * \brief List items and entities which are in the active region.
 * \param items (out) The interesting items.
 * \param regions A list of regions in which we take the items.
 */
void bear::universe::world::list_active_items
( item_set& items, const region_type& regions ) const
{
  item_set::const_iterator it;

  list_static_items( regions, items );

  for ( it=m_entities.begin(); it!=m_entities.end(); ++it )
    if ( item_in_regions(**it, regions) )
      items.insert( *it );
} // world::list_active_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Pick items overlapping a given position.
 * \param items (out) The interesting items.
 * \param pos The position where the items are searched.
 */
void bear::universe::world::pick_items
( item_set& items, const position_type& pos ) const
{
  rectangle_type region
    ( pos.x, pos.y, 2 * c_map_compression, pos.y - c_map_compression );

  if ( pos.x >= c_map_compression )
    region.position.x -= c_map_compression;
  else
    region.position.x = 0;

  if ( pos.y >= c_map_compression )
    region.position.y -= c_map_compression;
  else
    region.position.y = 0;

  item_set items_found;
  item_set::const_iterator it;
  region_type r;
  r.push_front(region);

  list_active_items( items_found, r );

  for ( it=items_found.begin(); it!=items_found.end(); ++it )
    {
      bear::universe::rectangle_type box = (*it)->get_bounding_box();

      if ( box.includes(pos) )
        items.insert( *it );
    }
} // world::pick_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Detection and correction collisions.
 * \param dynamic_items (out) Set of dynamic items.
 */
void bear::universe::world::detect_collision_all
( dynamic_item_set& dynamic_items )
{
  collision_detection coll_detection(dynamic_items);
   
  // each non phantom item with its neighbours which can move item
  bool modification = true;

  while ( modification )
    modification =
      detect_collision( dynamic_items, coll_detection, false, true );
  
  // each non phantom item with its neighbours which cannot move item
  detect_collision(dynamic_items, coll_detection, false, false);
  
  // each phantom item with its neighbours.
  detect_collision(dynamic_items, coll_detection, true, true);
  detect_collision(dynamic_items, coll_detection, true, false);
} // world::detect_collision_all()

/*----------------------------------------------------------------------------*/
/**
 * \brief Detection and correction collisions.
 * Return true if there is a modification.
 * \param dynamic_items (out) Set of dynamic items.
 * \param coll_detection (out) The instance of collision detection.
 * \param phantom_state The phantom state to consider.
 * \param can_move_item_state The can_move_item state to consider for each 
 * neighbour
 */
bool bear::universe::world::detect_collision
( dynamic_item_set& dynamic_items, collision_detection& coll_detection,
  bool phantom_state, bool can_move_item_state ) const
{
  dynamic_item_set::iterator_type it; // iterator on dynamic items map
  bool result = false;
  
  update_items_in_movement(dynamic_items, phantom_state);
  
  for(it=dynamic_items.begin(); it!=dynamic_items.end(); ++it)
    if ( ( !it->second.get_item()->is_fixed() ) &&
         ( it->first->is_phantom() == phantom_state) ) 
      {
        bool r = coll_detection.detect(it->second, can_move_item_state);
        result = result || r;
      }
  
  return result;
} // world::detect_collision()

/*----------------------------------------------------------------------------*/
/**
 * \brief For each living item, update the list of items which intersect 
 * the current dynamic item.
 * \param dynamic_items (out) Set of dynamic items.
 * \param phantom_state The phantom state to consider.
 */
void bear::universe::world::update_items_in_movement
( dynamic_item_set& dynamic_items, bool phantom_state ) const
{
  dynamic_item_set::iterator_type it; // iterator on dynamic items map
  item_set items;
  item_set::iterator it2; // iterator on entity set

  for(it=dynamic_items.begin(); it!=dynamic_items.end(); ++it)
    if ( ( ! it->first->is_fixed() ) && 
         ( it->first->is_phantom() == phantom_state ) ) 
      {
        items.clear();
        
        // add static items
        m_static_surfaces.get_area( it->first->get_bounding_box(), items );
        item_set::iterator it3;
        it->second.clear_neighbourhood();
        for ( it3 = items.begin(); it3 != items.end(); ++it3)
          it->second.add_neighbour(*it3);

        // add living item
        for ( it2=m_entities.begin(); it2!=m_entities.end(); ++it2 )
          if ( it->first->get_bounding_box().intersects
               ( (*it2)->get_bounding_box() ) )
            it->second.add_neighbour(*it2);       
      }
} // world::update_items_in_movement()

/*----------------------------------------------------------------------------*/
/**
 * \brief Search all interesting items in the active region
 *  and add dependent items.
 * \param regions The active regions.
 * \param static_items (out) List of static items.
 * \param living_items (out) List of living items.
 */
void bear::universe::world::search_interesting_items
( const region_type& regions, item_set& static_items, item_set& living_items)
{
  item_set::const_iterator it;

  // add static items of the active zone
  list_static_items( regions, static_items );
  
  // add global static items
  for (it=m_global_static_items.begin(); it!=m_global_static_items.end(); ++it)
    static_items.insert(*it);

  // add living item of the active zone and global living item
  for ( it=m_entities.begin(); it!=m_entities.end(); ++it )
    if ( (*it)->is_global() )
      living_items.insert(*it);
    else if ( item_in_regions(**it, regions) )
      living_items.insert(*it);
    
  // add dependent item
  stabilize_dependent_items(static_items, living_items);
} // world::search_interesting_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add and stabilize the lists of items.
 * \param static_items (out) List of static items.
 * \param living_items (out) List of living items.
 */
void bear::universe::world::stabilize_dependent_items
( item_set& static_items, item_set& living_items )
{
  unsigned int new_size = static_items.size() + living_items.size();  
  unsigned int last_size;
  item_set::const_iterator it;
  
  do
    {
      std::list<physical_item*> dependent_items;

      for ( it=living_items.begin(); it!=living_items.end(); ++it )
        (*it)->get_dependent_items(dependent_items);

      for ( it=static_items.begin(); it!=static_items.end(); ++it )
        (*it)->get_dependent_items(dependent_items);

      if( !dependent_items.empty() )
        add_dependent_items
          ( static_items, living_items, dependent_items );

      last_size = new_size;
      new_size = static_items.size() + living_items.size();
    }
  while (new_size != last_size);
} // world::stabilize_dependent_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add and stabilize the lists of items.
 * \param static_items (out) List of static items.
 * \param living_items (out) List of living items.
 * \param dependent_items List of dependent_items.
 */
void bear::universe::world::add_dependent_items
( item_set& static_items, item_set& living_items, 
  const std::list<physical_item*>& dependent_items )
{
  std::list<physical_item*>::const_iterator it;

  for( it=dependent_items.begin(); it!=dependent_items.end(); ++it )
    if ( *it != NULL )
      {
        base_entity* item = static_cast<base_entity*>(*it);
      
        if ( item->is_fixed() )
          static_items.insert(item);
        else 
          living_items.insert(item);
      }
} // world::add_dependent_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Create a dynamic item for each item.
 * \param dynamic_items (out) The set of dynamic items.
 * \param static_items The set of static item.
 * \param living_items The set of living item.
 */
void bear::universe::world::create_dynamic_items
( dynamic_item_set& dynamic_items, 
  const item_set& static_items, const item_set& living_items)
{
  item_set::const_iterator it;

  for ( it = static_items.begin(); it != static_items.end(); ++it ) 
    dynamic_items.add_item(*it);

  for ( it = living_items.begin(); it != living_items.end(); ++it ) 
    dynamic_items.add_item(*it);
} // world::create_dynamic_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Call the progress() method on some items.
 * \param items The items on which we call the method.
 * \param elapsed_time Elapsed time since the last call.
 */
void bear::universe::world::progress_items
( const item_set& items, time_type elapsed_time ) const
{
  item_set::const_iterator it;

  for( it=items.begin(); it!=items.end(); ++it )
    (*it)->progress( elapsed_time );
} // world::progress_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Update positions of stored items and check collisions.
 * \param elapsed_time Elasped time since the last progress.
 * \param dynamic_items (out) The set of dynamic items to progress.
 */
void bear::universe::world::progress
( time_type elapsed_time, dynamic_item_set& dynamic_items )
{
  // progress the positions
  dynamic_item_set::iterator_type it;

  for(it=dynamic_items.begin(); it!=dynamic_items.end(); ++it)
    {
      if ( ( ( it->first->get_mass() != 0) || 
             it->first->has_forced_movement() )
           && ( !it->first->is_fixed() ) )
        it->first->move( elapsed_time );

      it->first->clear_contacts();
    }
  
  apply_links(dynamic_items);
} // world::progress()

/*----------------------------------------------------------------------------*/
/**
 * \brief Apply the links forces between items.
 * \param dynamic_items The set of dynamic items to progress. 
 */
void bear::universe::world::apply_links
(const dynamic_item_set& dynamic_items) const
{
  claw::avl<base_link*> links;
  claw::avl<base_link*>::const_iterator it;
  dynamic_item_set::const_iterator_type item;

  for (item=dynamic_items.const_begin(); 
       item!=dynamic_items.const_end(); ++item)
    links.insert( item->first->links_begin(), item->first->links_end() );

  for( it=links.begin(); it!=links.end(); ++it )
    (*it)->adjust();
} // world::apply_links()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell items if they leave the active region.
 * \param regions A list of regions in which we take the items.
 * \param items The items to verify.
 */
void bear::universe::world::active_region_traffic
( const region_type& regions, const item_set& items )
{
  item_set::const_iterator it;

  for( it=m_last_interesting_items.begin(); it!=m_last_interesting_items.end();
       ++it )
    if ( m_entities.find(*it) != m_entities.end() )
      if ( items.find(*it) == items.end() )
        (*it)->left_active_region();
        
  m_last_interesting_items = items;
} // world::active_region_traffic()

/*----------------------------------------------------------------------------*/
/**
 * \brief List static items which are in the active region.
 * \param items (out) The interesting items.
 * \param regions A list of regions in which we take the items.
 */
void bear::universe::world::list_static_items
( const region_type& regions, item_set& items ) const
{
  region_type::const_iterator it;

  for (it=regions.begin(); it!=regions.end(); ++it)
    m_static_surfaces.get_area( *it, items);
} // world::list_static_items()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if an item is in a region.
 * \param item The item to check.
 * \param regions A list of regions in which search the item.
 */
bool bear::universe::world::item_in_regions
( const base_entity& item, const region_type& regions ) const
{
  return regions.intersects( item.get_bounding_box() );
} // world::item_in_regions()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add an entity in the world.
 * \param who The entity to add.
 */
void bear::universe::world::add(base_entity* const& who)
{
  who->set_owner(*this);
  m_entities.insert( who );
} // world::add()

/*----------------------------------------------------------------------------*/
/**
 * \brief Remove an entity from the world.
 * \param who The entity to remove.
 */
void bear::universe::world::remove(base_entity* const& who)
{
  if ( m_entities.find(who) != m_entities.end() ) 
    m_entities.erase(who);
} // world::remove()

