/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file physical_item.hpp
 * \brief The common interface for all the physical items living in our
 *        universe.
 * \author Julien Jorge
 */
#ifndef __UNIVERSE_PHYSICAL_ITEM_HPP__
#define __UNIVERSE_PHYSICAL_ITEM_HPP__

#include <claw/math.hpp>

#include <list>

#include "universe/item_handle.hpp"
#include "universe/physical_item_state.hpp"
#include "universe/zone.hpp"

namespace bear
{
  namespace universe
  {
    class forced_movement;
    class collision_info;
    class base_link;
    class world;

    /**
     * \brief The common interface for all the physical items living in our
     *        universe.
     * \author Julien Jorge
     */
    class UNIVERSE_EXPORT physical_item :
      public physical_item_state
    {
    public:
      /** \brief The type of the class that stores our fields. */
      typedef physical_item_state super;

      /** \brief The type of the list in which we store the links. */
      typedef std::list<base_link*> link_list_type;

      /** \brief Const iterator on the list of links. */
      typedef link_list_type::const_iterator const_link_iterator;

    public:
      physical_item();
      physical_item( const physical_item& that );
      virtual ~physical_item();

      forced_movement& get_forced_movement();
      void set_forced_movement( const forced_movement& m );
      bool has_forced_movement() const;
      void clear_forced_movement();

      virtual void move( time_type elapsed_time );
      virtual void collision( collision_info& info );

      // public only for world
      void set_owner(world& owner);
      // -end- public only for world

      // public only for base_link
      void add_link( base_link& link );
      void remove_link( base_link& link );
      // -end- public only for base_link

      const_link_iterator links_begin() const;
      const_link_iterator links_end() const;

      // public only for item_handle
      void add_handle( item_handle* h );
      void remove_handle( item_handle* h );
      // -end- public only for item_handle

      virtual void get_dependent_items( std::list<physical_item*>& items );

    protected:
      bool has_owner() const;
      world& get_owner() const;

      void default_move( time_type elapsed_time );
      bool default_collision( collision_info& info );

      bool collision_align( collision_info& info );

      bool collision_align_left( collision_info& info );
      bool collision_align_right( collision_info& info );
      bool collision_align_top( collision_info& info );
      bool collision_align_bottom( collision_info& info );

      bool
      collision_align_left( physical_item& that, const position_type& pos );
      bool
      collision_align_right( physical_item& that, const position_type& pos );
      bool collision_align_top( physical_item& that, const position_type& pos );
      bool
      collision_align_bottom( physical_item& that, const position_type& pos );

    private:
      bool
      collision_align_at( physical_item& that, const position_type& pos ) const;

      void remove_all_links();

      void update_position( time_type elapsed_time );
      void adjust_cinetic();

    private:
      /** \brief Forced movement applied to this item, NULL if none. */
      forced_movement* m_forced_movement;

      /** \brief The links concerning the item. */
      link_list_type m_links;

      /** \brief The handles on me. */
      std::list<item_handle*> m_handles;

      /** \brief The world in which this item lives. */
      world* m_owner;

    }; // class physical_item
  } // namespace universe
} // namespace bear

#endif // __UNIVERSE_PHYSICAL_ITEM_HPP__
