/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file animation.cpp
 * \brief Implementation of the bear::visual::animation class.
 * \author Julien Jorge
 */
#include "visual/animation.hpp"
#include <claw/assert.hpp>

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
bear::visual::animation::animation()
  : m_time(0)
{

} // animation::animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param images Names of the frames images.
 * \param d The duration of the frames.
 * \pre images.size() > 0
 * \pre images.size() == d.size()
 * \post loops == 0, loop_back == false
 * \remark The sprites pointers passed as arguments will be deleted in the
 *         destructor.
 */
bear::visual::animation::animation
( const std::vector<sprite>& images, const std::vector<double>& d )
  : sprite_sequence(images), m_duration(d), m_time(0)
{
  CLAW_PRECOND( images.size() == d.size() );
} // animation::animation()

/*----------------------------------------------------------------------------*/
/**
 * \brief Come back to first frame.
 */
void bear::visual::animation::reset()
{
  sprite_sequence::reset();
  m_time = 0;
} // animation::reset()

/*----------------------------------------------------------------------------*/
/**
 * \brief Go forward in the animation.
 * \param t Go this dutation forward.
 * \pre t >= 0
 */
void bear::visual::animation::next( double t )
{
  CLAW_PRECOND( t >= 0 );

  if ( !is_finished() )
    {
      m_time += t;

      while ( (m_time >= m_duration[get_current_index()])
              && !sprite_sequence::is_finished() )
        {
          sprite_sequence::next();
          m_time -= m_duration[get_current_index()];
        }
    }
} // animation::next()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if we have reached the last frame and if it is completely played.
 */
bool bear::visual::animation::is_finished() const
{
  return sprite_sequence::is_finished()
    && (m_time >= m_duration[get_current_index()]);
} // animation::is_finished()
