/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file visual/code/scene_sprite.cpp
 * \brief Implementation of the visual::scene_sprite.
 * \author Julien Jorge
 */
#include <climits>
#include <limits>

#include "visual/scene_sprite.hpp"

#include "visual/scene_element.hpp"

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param pos On screen position of the sprite.
 * \param s The sprite to draw.
 * \param a Rotate the sprite with this angle (in radians and in trigonometric
 *        direction). The center of the rotation is the center of the box
 *       (pos, s.get_size).
 */
bear::visual::scene_sprite::scene_sprite
( const position_type& pos, const sprite& s, double a )
  : m_position(pos), m_sprite(s), m_angle(a)
{

} // scene_sprite::scene_sprite()

/*----------------------------------------------------------------------------*/
/**
 * \brief Allocate a copy of this instance.
 */
bear::visual::base_scene_element* bear::visual::scene_sprite::clone() const
{
  return new scene_sprite(*this);
} // scene_sprite::clone()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get a rectangle where the sprite is completely opaque.
 */
bear::visual::rectangle_type bear::visual::scene_sprite::get_opaque_box() const
{
  if ( m_sprite.has_transparency() || (m_angle != 0) )
    return rectangle_type(0, 0, 0, 0);
  else
    return get_bounding_box();
} // scene_sprite::get_opaque_box()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get a rectangle bounding the sprite.
 */
bear::visual::rectangle_type
bear::visual::scene_sprite::get_bounding_box() const
{
  if ( m_angle != 0 ) 
    {
     position_type left_top( std::numeric_limits<coordinate_type>::max(),
                              std::numeric_limits<coordinate_type>::max() );

      position_type right_bottom( 0,0 );
      
      position_type center( m_position.x + m_sprite.width() / 2,
                            m_position.y + m_sprite.height() / 2);
      
      update_side_box( position_type(m_position.x,m_position.y),
                       center, left_top, right_bottom);

      update_side_box( position_type(m_position.x+m_sprite.width(),
                                     m_position.y),
                       center, left_top, right_bottom);

      update_side_box( position_type(m_position.x,
                                     m_position.y+m_sprite.height()),
                       center, left_top, right_bottom);

      update_side_box( position_type(m_position.x+m_sprite.width(),
                                     m_position.y+m_sprite.height()),
                       center, left_top, right_bottom);

      return rectangle_type( left_top.x, left_top.y, 
                             right_bottom.x - left_top.x,
                             right_bottom.y - left_top.y );
    }
  else
    return rectangle_type
      ( m_position.x, m_position.y, m_sprite.width(), m_sprite.height() );
  
} // scene_sprite::get_bounding_box()

/*----------------------------------------------------------------------------*/
/**
 * \brief Split the sprite in sub sprites.
 * \param boxes The boxes describing how to split the sprite.
 * \param output The resulting sprites (they are inserted to the front).
 */
void bear::visual::scene_sprite::burst
( const rectangle_list& boxes, scene_element_list& output ) const
{
  output.push_front( scene_element(*this) );
} // scene_sprite::burst()

/*----------------------------------------------------------------------------*/
/**
 * \brief Render the sprite on a screen.
 * \param scr The screen on which we render the sprite.
 */
void bear::visual::scene_sprite::render( base_screen& scr ) const
{
  scr.render(m_position, m_sprite, m_angle);
} // scene_sprite::render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Update sides of bounding box when we apply a rotation.
 *
 * \param pos The point to rotate.
 * \param center The center of the rotation.
 * \param left_top (out) The left_top position of the bounding box.
 * \param right_bottom (out) The right_bottom position of the bounding box.
 */
void bear::visual::scene_sprite::update_side_box
( const position_type& pos, const position_type& center,
  position_type& left_top, position_type& right_bottom) const
{
  bear::visual::position_type result(center);
  
  // According to the direction of y axis, we take the opposite angle. 
  GLdouble new_a = -m_angle;

  result.x +=
    (pos.x - center.x) * cos(new_a) + (pos.y - center.y) * sin(new_a);
  result.y +=
    -(pos.x - center.x) * sin(new_a) + (pos.y - center.y) * cos(new_a);

  if ( result.x < left_top.x )
    left_top.x = result.x;

  if ( result.y < left_top.y )
    left_top.y = result.y;

  if ( result.x > right_bottom.x )
    right_bottom.x = result.x;

  if ( result.y > right_bottom.y )
    right_bottom.y = result.y;
} // scene_sprite::update_side_box()
