/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file image.cpp
 * \brief Implementation of the bear::visual::image class.
 * \author Julien Jorge
 */
#include <climits>
#include <limits>

#include "visual/sdl_image.hpp"

#include <claw/exception.hpp>
#include <claw/assert.hpp>

/*----------------------------------------------------------------------------*/
std::map<SDL_Surface*, unsigned int>
bear::visual::sdl_image::s_texture_references;

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor with a claw::graphic::sdl_image object.
 * \param data The image to copy.
 */
bear::visual::sdl_image::sdl_image(const claw::graphic::image& data)
  : m_texture(NULL), m_has_transparency(false)
{
  create_texture( data );
} // sdl_image::sdl_image() [claw::graphic::sdl_image]

/*----------------------------------------------------------------------------*/
/**
 * \brief Copy constructor.
 * \param that The image to copy from.
 */
bear::visual::sdl_image::sdl_image(const sdl_image& that)
  : m_texture(that.m_texture), m_size(that.m_size),
    m_has_transparency(that.has_transparency())
{
  register_texture();
} // sdl_image::sdl_image() [copy constructor]

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
bear::visual::sdl_image::~sdl_image()
{
  release_texture();
} // sdl_image::~sdl_image()

/*----------------------------------------------------------------------------*/
/**
 * \brief Instanciate a copy of the image.
 */
bear::visual::base_image* bear::visual::sdl_image::clone() const
{
  return new sdl_image(*this);
} // sdl_image::clone()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the SDL texture's surface.
 */
const SDL_Surface* bear::visual::sdl_image::get_surface() const
{
  return m_texture;
} // sdl_image::get_surface()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get image's size.
 */
claw::math::coordinate_2d<unsigned int> bear::visual::sdl_image::size() const
{
  return m_size;
} // sdl_image::size()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if the image has transparent pixels.
 */
bool bear::visual::sdl_image::has_transparency() const
{
  return m_has_transparency;
} // sdl_image::has_transparency()

/*----------------------------------------------------------------------------*/
/**
 * \brief Assigment operator.
 */
bear::visual::sdl_image&
bear::visual::sdl_image::operator=( const bear::visual::sdl_image& that )
{
  if (m_texture != that.m_texture)
    {
      release_texture();

      m_texture = that.m_texture;
      m_size = that.m_size;

      register_texture();
    }

  return *this;
} // sdl_image::size()

/*----------------------------------------------------------------------------*/
/**
 * \brief Create a SDL texture.
 * \param data The image of the texture.
 */
void bear::visual::sdl_image::create_texture( const claw::graphic::image& data )
{
  m_size.x = data.width();
  m_size.y = data.height();

  Uint32* pixels = new Uint32[m_size.x * m_size.y];
  Uint32* p = pixels;
  claw::graphic::image::const_iterator it;
  const claw::graphic::pixel32::component_type m =
    std::numeric_limits<claw::graphic::pixel32::component_type>::max();

  for (it=data.begin(); it!=data.end(); ++it, ++p)
    {
      *p = it->pixel;
      m_has_transparency = m_has_transparency || (it->components.alpha != m);
    }

  const claw::graphic::pixel32 r_mask(m, 0, 0, 0);
  const claw::graphic::pixel32 g_mask(0, m, 0, 0);
  const claw::graphic::pixel32 b_mask(0, 0, m, 0);
  const claw::graphic::pixel32 a_mask(0, 0, 0, m);

  m_texture = SDL_CreateRGBSurfaceFrom
    ( pixels, m_size.x, m_size.y, 32, m_size.x * sizeof(Uint32), r_mask.pixel,
      g_mask.pixel, b_mask.pixel, a_mask.pixel );

  register_texture();
} // sdl_image::create_texture()

/*----------------------------------------------------------------------------*/
/**
 * \brief Register an OpenGL texture.
 * \param if The identifier of the texture.
 */
void bear::visual::sdl_image::register_texture() const
{
  std::map<SDL_Surface*, unsigned int>::iterator it =
    s_texture_references.find(m_texture);

  if ( it == s_texture_references.end() )
    s_texture_references.insert( std::make_pair(m_texture, 1) );
  else
    ++it->second;
} // sdl_image::register_texture()

/*----------------------------------------------------------------------------*/
/**
 * \brief Release an OpenGL texture.
 * \param if The identifier of the texture.
 */
void bear::visual::sdl_image::release_texture()
{
  CLAW_PRECOND
    ( s_texture_references.find(m_texture) != s_texture_references.end() );

  std::map<SDL_Surface*, unsigned int>::iterator it =
    s_texture_references.find(m_texture);

  --it->second;

  if ( it->second == 0 )
    {
      delete[] (Uint32*)it->first->pixels;
      SDL_FreeSurface(it->first);
      s_texture_references.erase(it);
    }
} // sdl_image::release_texture()
