/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file sdl_screen.cpp
 * \brief Implementation of the bear::visual::sdl_screen class.
 * \author Julien Jorge
 */
#include "visual/sdl_screen.hpp"

#include "visual/sdl_image.hpp"

#include <claw/exception.hpp>
#include <SDL/SDL.h>
#include <limits>

/*----------------------------------------------------------------------------*/
/**
 * \brief Global initializations common to all sdl_screens. Must be called at
 *        the begining of your program.
 */
void bear::visual::sdl_screen::initialize()
{
  if ( !SDL_WasInit(SDL_INIT_VIDEO) )
    if ( SDL_InitSubSystem(SDL_INIT_VIDEO) != 0 )
      throw CLAW_EXCEPTION( SDL_GetError() );
} // sdl_screen::initialize()

/*----------------------------------------------------------------------------*/
/**
 * \brief Sdlobal uninitializations common to all sdl_screens. Must be called at the
 *        end of your program.
 */
void bear::visual::sdl_screen::release()
{
  SDL_QuitSubSystem(SDL_INIT_VIDEO);
} // sdl_screen::release()

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param size Size of the sdl_screen.
 * \param title The title of the window created.
 * \param full Tell if the window is full sdl_screen or not.
 */
bear::visual::sdl_screen::sdl_screen
( const claw::math::coordinate_2d<unsigned int>& size,
  const std::string& title, bool full )
  : m_size(size)
{
  fullscreen(full);

  SDL_WM_SetCaption( title.c_str(), NULL );
} // sdl_screen::sdl_screen() [constructor]

/*----------------------------------------------------------------------------*/
/**
 * \brief Set a new dimension for the resulting projection (doesn't change
 *        the sdl_screen's size.
 * \param width Target's width.
 * \param height Target's height.
 */
void
bear::visual::sdl_screen::resize_view(unsigned int width, unsigned int height)
{
  m_size.set(width, height);
  fullscreen(false);
} // sdl_screen::resize_view()

/*----------------------------------------------------------------------------*/
/**
 * \brief Turn fullscreen mode on/off.
 * \param b Tell if we want a fullsdl_screen mode.
 */
void bear::visual::sdl_screen::fullscreen( bool b )
{
  Uint32 flags = SDL_HWSURFACE | SDL_RESIZABLE | SDL_DOUBLEBUF;

  if (b)
    flags |= SDL_FULLSCREEN;

  SDL_Surface* s = SDL_SetVideoMode( m_size.x, m_size.y, 32, flags );

  if (!s)
    throw CLAW_EXCEPTION( SDL_GetError() );
} // sdl_screen::fullsdl_screen()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the size of the sdl_screen.
 */
claw::math::coordinate_2d<unsigned int>
bear::visual::sdl_screen::get_size() const
{
  return m_size;
} // sdl_screen::get_size()

/*----------------------------------------------------------------------------*/
/**
 * \brief Initialize the rendering process.
 */
void bear::visual::sdl_screen::begin_render()
{
  SDL_FillRect( SDL_GetVideoSurface(), NULL, 0 );
} // sdl_screen::begin_render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Draw a sprite on the sdl_screen.
 * \param pos On sdl_screen position of the sprite.
 * \param s The sprite to draw.
 * \param a Rotate the sprite with this ansdle (in radians and in trigonometric
 *        direction). The center of the rotation is the center of the box
 *       (pos, s.get_size).
 */
void bear::visual::sdl_screen::render
( const claw::math::coordinate_2d<int>& pos, const sprite& s, double a )
{
  const sdl_image* c_impl =
    static_cast<const sdl_image*>(s.get_image().get_impl());

  SDL_Rect s_rect, d_rect;

  s_rect.x = s.clip_rectangle().position.x;
  s_rect.y = s.clip_rectangle().position.y;
  s_rect.w = s.clip_rectangle().width;
  s_rect.h = s.clip_rectangle().height;

  d_rect.x = pos.x;
  d_rect.y = pos.y;
  d_rect.w = s.width();
  d_rect.h = s.height();

  const SDL_Surface* tex = c_impl->get_surface();
  const SDL_PixelFormat* fmt = tex->format;

  SDL_Surface* src = 
    SDL_CreateRGBSurfaceFrom
    ( tex->pixels, tex->w, tex->h, fmt->BitsPerPixel, tex->pitch, fmt->Rmask,
      fmt->Gmask, fmt->Bmask, fmt->Amask );
  
  SDL_SetAlpha
    ( src, SDL_SRCALPHA, Uint8(s.get_alpha_blend() * SDL_ALPHA_OPAQUE) );
  SDL_BlitSurface( src, &s_rect, SDL_GetVideoSurface(), &d_rect );

  SDL_FreeSurface(src);
} // sdl_screen::render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Stop the rendering process.
 */
void bear::visual::sdl_screen::end_render()
{
  SDL_Flip(SDL_GetVideoSurface());
} // sdl_screen::end_render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Draw a line.
 * \param color The color of the line.
 * \param p The points of the line.
 * \param w The width of the line.
 */
void bear::visual::sdl_screen::draw_line
( const claw::graphic::pixel32& color, const std::vector<position_type>& p,
  double w )
{

} // sdl_screen::draw_line()

/*----------------------------------------------------------------------------*/
/**
 * \brief Do a sdl_screen shot.
 * \param img The image in which we save the content of the sdl_screen.
 */
void bear::visual::sdl_screen::shot( claw::graphic::image& img ) const
{
  
} // sdl_screen::shot()
