/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file invisible_slope.cpp
 * \brief Implementation of the bear::slope class.
 * \author Julien Jorge
 */
#include "generic_items/invisible_slope.hpp"

#include "universe/collision_info.hpp"
#include "engine/export.hpp"

BASE_ITEM_EXPORT( invisible_slope, bear )

/*----------------------------------------------------------------------------*/
const bear::universe::coordinate_type bear::invisible_slope::s_line_width = 10;

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
bear::invisible_slope::invisible_slope()
  : m_steepness(0), m_opposite_side_is_active(false),
    m_left_side_is_active(false), m_right_side_is_active(false)
{

} // invisible_slope::invisible_slope()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set a field of type \c <real>.
 * \param name The name of the field to set.
 * \param value The new value of the field.
 */
bool
bear::invisible_slope::set_real_field( const std::string& name, double value )
{
  bool result = true;

  if ( name == "steepness" )
    m_steepness = value;
  else
    result = super::set_real_field(name, value);

  return result;
} // invisible_slope::set_real_field()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set a field of type \c bool.
 * \param name The name of the field to set.
 * \param value The new value of the field.
 */
bool
bear::invisible_slope::set_bool_field( const std::string& name, bool value )
{
  bool result = true;

  if ( name == "opposite_side_is_active" )
    m_opposite_side_is_active = value;
  else if ( name == "left_side_is_active" )
    m_left_side_is_active = value;
  else if ( name == "right_side_is_active" )
    m_right_side_is_active = value;
  else
    result = super::set_u_integer_field(name, value);

  return result;
} // invisible_slope::set_bool_field()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if the item is correctly initialized.
 */
bool bear::invisible_slope::is_valid() const
{
  return (m_steepness != 0) && super::is_valid();
} // invisible_slope::is_valid()

/*----------------------------------------------------------------------------*/
/**
 * \brief Initialize the item.
 */
void bear::invisible_slope::build()
{
  super::build();

  m_line = line_type(get_left(), get_top(), get_width(), m_steepness);

  if ( m_steepness < 0 )
    m_line.origin.y += -m_steepness;
} // invisible_slope::build()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align the other item.
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
void bear::invisible_slope::collision_as_slope
( engine::base_item& that, universe::collision_info& info )
{
  double f(1);
  bool align_as_block(false);
  bool align_as_slope(false);

  switch( info.get_collision_side() )
    {
    case universe::zone::bottom_zone:
      f = get_bottom_friction();
      align_as_block = m_opposite_side_is_active;
      break;
    case universe::zone::top_zone:
      f = get_top_friction();
      align_as_slope = true;
      break;
    case universe::zone::middle_left_zone:
      f = get_left_friction();
      if ( check_left_contact_as_slope(that, info) )
        align_as_slope = true;
      else
        align_as_block = m_left_side_is_active;
      break;
    case universe::zone::middle_right_zone:
      f = get_right_friction();
      if ( check_right_contact_as_slope(that, info) )
        align_as_slope = true;
      else
        align_as_block = m_right_side_is_active;
      break;
    case universe::zone::middle_zone:
      f = get_top_friction();
      if ( check_bottom_above_ground(that, info) )
        align_as_slope = true;
      break;
    default: { CLAW_ASSERT( false, "Invalid collision side." ); }
    }

  bool aligned(false);

  if ( align_as_slope )
    aligned = align_on_ground(that, info);
  else if ( align_as_block )
    aligned = default_collision(info);
  else
    aligned = align_nearest_edge(that, info);

  if (aligned)
    {
      that.set_contact_friction(f);
      z_shift(that);
    }
} // invisible_slope::collision_as_slope()

/*----------------------------------------------------------------------------*/
/**
 * \brief Call collision_as_slope().
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
void bear::invisible_slope::collision
( engine::base_item& that, universe::collision_info& info )
{
  collision_as_slope(that, info);
} // invisible_slope::collision()

/*----------------------------------------------------------------------------*/
/**
 * \brief Check if an item colliding on the left is aligned on the ground.
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
bool bear::invisible_slope::check_left_contact_as_slope
( engine::base_item& that, universe::collision_info& info ) const
{
  bool result = false;

  // the slope goes from top right to bottom left
  if ( m_steepness < 0 )
    result = info.get_top_left_on_contact().y + that.get_height()
      <= m_line.y_value( get_left() ) + s_line_width;

  return result;
} // invisible_slope::check_left_contact_as_slope()

/*----------------------------------------------------------------------------*/
/**
 * \brief Check if an item colliding on the right is aligned on the ground.
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
bool bear::invisible_slope::check_right_contact_as_slope
( engine::base_item& that, universe::collision_info& info ) const
{
  bool result = false;

  // the slope goes from top left to bottom right
  if ( m_steepness > 0 )
    result = info.get_top_left_on_contact().y + that.get_height()
      <= m_line.y_value( get_right() ) + s_line_width;

  return result;
} // invisible_slope::check_right_contact_as_slope()

/*----------------------------------------------------------------------------*/
/**
 * \brief Check if the bottom of the other item was above the ground at
 *        collision time.
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
bool bear::invisible_slope::check_bottom_above_ground
( engine::base_item& that, universe::collision_info& info ) const
{
  const universe::position_type pos
    (info.other_previous_state().get_bottom_middle());

  return pos.y <= m_line.y_value(pos.x) + s_line_width;
} // invisible_slope::check_bottom_above_ground()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align \a that on the ground line.
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
bool bear::invisible_slope::align_on_ground
( engine::base_item& that, const universe::collision_info& info )
{
  bool result = false;

  const universe::coordinate_type pos_x
    ( info.get_top_left_on_contact().x + that.get_width() / 2 );

  if ( (pos_x >= get_left()) && (pos_x <= get_right())
       && item_crossed_up_down(that, info) )
    {
      const universe::position_type pos
        ( info.get_top_left_on_contact().x,
          m_line.y_value(pos_x) - that.get_height() );

      if ( collision_align_top(that, pos) )
        {
          result = true;

          universe::speed_type speed( that.get_speed().x, 0 );
          universe::force_type accel( that.get_acceleration().x, 0 );

          /*if ( (m_line.direction.y > 0) && (speed.x < 0)
               || (m_line.direction.y < 0) && (speed.x > 0) )
            if ( m_line.direction.x != 0 )
              {
                double ratio =
                  std::abs(m_line.direction.y) / ( 2 * m_line.direction.x );

                speed.x *= 1.0 - std::min(ratio, 1.0);
                }*/

          that.set_speed(speed);
          that.set_acceleration(accel);
        }
    }

  return result;
} // invisible_slope::align_on_ground()

/*----------------------------------------------------------------------------*/
/**
 * \brief Align \a that on the nearest vertical edge.
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
bool bear::invisible_slope::align_nearest_edge
( engine::base_item& that, universe::collision_info& info )
{
  bool result = false;

  if ( info.reference_previous_state().get_center_of_mass().x
       <= info.other_previous_state().get_center_of_mass().x )
    {
      if (m_right_side_is_active)
        result = collision_align_right(info);
    }
  else if (m_left_side_is_active)
    result = collision_align_left(info);

  return result;
} // invisible_slope::align_nearest_edge()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell if the other item has crossed the surface following a top to down
 *        direction.
 * \param that The other item in the collision.
 * \param info Informations on the collision.
 */
bool bear::invisible_slope::item_crossed_up_down
( engine::base_item& that, const universe::collision_info& info ) const
{
  bool result = false;

  if ( that.get_bottom()
       >= m_line.y_value(that.get_center_of_mass().x) - s_line_width )
    {
      const universe::position_type other_prev_bottom
        ( info.other_previous_state().get_center_of_mass().x,
          info.other_previous_state().get_bottom() );
      const universe::coordinate_type y_right = m_line.y_value( get_right() );

      if ( other_prev_bottom.x < get_left() )
        result = other_prev_bottom.y <= m_line.origin.y + s_line_width;
      else if ( other_prev_bottom.x > get_right() )
        result = other_prev_bottom.y <= y_right + s_line_width;
      else
        result = other_prev_bottom.y
          <= m_line.y_value(other_prev_bottom.x) + s_line_width;
    }

  return result;
} // invisible_slope::item_crossed_up_down()

