/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file invisible_straight_slope.cpp
 * \brief Implementation of the bear::invisible_straight_slope class.
 * \author Julien Jorge
 */
#include "generic_items/invisible_straight_slope.hpp"

#include "universe/collision_info.hpp"
#include "engine/export.hpp"

BASE_ITEM_EXPORT( invisible_straight_slope, bear )

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
bear::invisible_straight_slope::invisible_straight_slope()
  : m_margin(10), m_top_side_is_active(false),
    m_bottom_side_is_active(false)
{

} // invisible_straight_slope::invisible_straight_slope()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set a field of type bool.
 * \param name The name of the field.
 * \param value The new value of the field.
 * \return false if the field "name" is unknow, true otherwise.
 */
bool bear::invisible_straight_slope::set_bool_field
( const std::string& name, bool value )
{
  bool ok(true);

  if (name == "top_side_is_active")
    m_top_side_is_active = value;
  else if (name == "bottom_side_is_active")
    m_bottom_side_is_active = value;
  else
    ok = super::set_bool_field(name, value);

  return ok;
} // invisible_straight_slope::set_bool_field()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set a field of type \c <real>.
 * \param name The name of the field to set.
 * \param value The new value of the field.
 */
bool bear::invisible_straight_slope::set_real_field
( const std::string& name, double value )
{
  bool result = true;

  if ( name == "margin" )
    m_margin = value;
  else
    result = super::set_real_field(name, value);

  return result;
} // invisible_straight_slope::set_real_field()

/*----------------------------------------------------------------------------*/
/**
 * \brief Check if that the center of the other item is included in the bounds
 * of \a this before processing the collision.
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
void bear::invisible_straight_slope::collision_check_center_included
( engine::base_item& that, universe::collision_info& info )
{
  double f(1);

  const universe::coordinate_type pos_x
    ( info.get_top_left_on_contact().x + that.get_width() / 2 );

  if ( !that.is_phantom() && (pos_x >= get_left()) && (pos_x <= get_right()) )
    {
      const universe::zone::position z = info.get_collision_side();
      const universe::coordinate_type pos_top
        ( info.get_top_left_on_contact().y );
      const universe::coordinate_type pos_bottom
        ( info.get_top_left_on_contact().y + that.get_height() );
      bool align = false;

      if ( (z == universe::zone::top_zone)
           || (pos_bottom <= get_top() + m_margin) )
        {
          if (m_top_side_is_active)
            {
              align = collision_align_top(info);
              f = get_top_friction();
            }
        }
      else if ( (z == universe::zone::bottom_zone)
                || (pos_top >= get_bottom() - m_margin) )
        {
          if (m_bottom_side_is_active)
            {
              align = collision_align_bottom(info);
              f = get_bottom_friction();
            }
        }

      if ( align )
        {
          that.set_contact_friction(f);
          z_shift(that);
        }
    }
} // invisible_straight_slope::collision_check_center_include()

/*----------------------------------------------------------------------------*/
/**
 * \brief Call collision_check_center_included().
 * \param that The other item of the collision.
 * \param info Some informations about the collision.
 */
void bear::invisible_straight_slope::collision
( engine::base_item& that, universe::collision_info& info )
{
  collision_check_center_included(that, info);
} // invisible_straight_slope::collision()
