/*
  Plee the Bear

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [PTB] in the subject of your mails.
*/
/**
 * \file frame_password.cpp
 * \brief Implementation of the ptb::frame_password class.
 * \author Julien Jorge
 */
#include <climits>
#include <limits>

#include "ptb/frame/frame_password.hpp"

#include "engine/game.hpp"
#include "engine/string_base.hpp"

#include "ptb/config_file.hpp"

#include <claw/string_algorithm.hpp>

/*----------------------------------------------------------------------------*/
const unsigned int ptb::frame_password::s_margin = 10;

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param owning_layer The layer owning the window.
 */
ptb::frame_password::frame_password( windows_layer* owning_layer )
  : frame(owning_layer)
{
  create_controls();
} // frame_password::frame_password()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell that a key has been pressed.
 * \param key The code of the key.
 */
bool ptb::frame_password::on_key_press( bear::input::keyboard::key_code key )
{
  bool result = true;

  switch( key )
    {
    case bear::input::keyboard::kc_escape:
      on_back();
      break;
    case bear::input::keyboard::kc_new_line:
    case bear::input::keyboard::kc_keypad_enter:
      validate();
      break;
    default:
      result = false;
    }

  return result;
} // frame_password::on_key_press()

/*----------------------------------------------------------------------------*/
/**
 * \brief Tell that a joystick button has been pressed.
 * \param button The code of the button.
 * \param joy_index The index of the joytick.
 */
bool ptb::frame_password::on_button_press
( bear::input::joystick::joy_code button, unsigned int joy_index )
{
  bool result = true;

  switch( button )
    {
    case bear::input::joystick::jc_axis_left:
    case bear::input::joystick::jc_axis_right:
    case bear::input::joystick::jc_axis_up:
    case bear::input::joystick::jc_axis_down:
      result = false;
      break;
    case bear::input::joystick::jc_button_1:
      on_back();
      break;
    default:
      validate();
      break;
    }

  return result;
} // frame_password::on_button_press()

/*----------------------------------------------------------------------------*/
/**
 * \brief Validate the selected action.
 */
void ptb::frame_password::validate()
{
  std::vector<std::string> command;
  std::string text( m_password->get_text() );
  m_password->set_text("");

  claw::text::trim(text);
  claw::text::squeeze(text, " ");
  claw::text::split(command, text, ' ');

  if ( !command.empty() )
    execute_command(command);
} // frame_password::validate()

/*----------------------------------------------------------------------------*/
/**
 * \brief Create the static text components.
 */
void ptb::frame_password::create_controls()
{
  bear::gui::static_text::font_type font = frame::get_font();

  create_input( font );
  create_back( font );

  set_size( m_back->right() + s_margin, m_back->bottom() + s_margin );
} // frame_password::create_controls()

/*----------------------------------------------------------------------------*/
/**
 * \brief Create the text input control.
 * \param f The font to use for the text.
 */
void ptb::frame_password::create_input( bear::gui::static_text::font_type f )
{
  const claw::graphic::pixel32::component_type full_color =
    std::numeric_limits<claw::graphic::pixel32::component_type>::max();

  m_password =
    new bear::gui::text_input
    ( &get_frame(), f, claw::graphic::pixel32(full_color, 0, 0, full_color) );

  m_password->set_position( s_margin, s_margin );
  m_password->set_width(300);
  m_password->set_border_color( get_border_color() );
} // frame_password::create_input()

/*----------------------------------------------------------------------------*/
/**
 * \brief Create the "back" buttons.
 * \param f The font to use for the texts.
 */
void ptb::frame_password::create_back( bear::gui::static_text::font_type f )
{
  std::string text("back");

  m_back =
    new button_with_text( *this, std::mem_fun(&frame_password::on_back), this );

  m_back->set_font( f );
  m_back->set_auto_size( true );
  bear::engine::string_base::get_instance().get_string(text);
  m_back->set_text(text);

  const unsigned int y = m_password->bottom() + s_margin;

  if ( s_margin + m_back->width() > m_password->right() )
    m_back->set_position( s_margin, y );
  else
    m_back->set_position( m_password->right() - m_back->width(), y );
} // frame_language::create_back()

/*----------------------------------------------------------------------------*/
/**
 * \brief Exit.
 */
bool ptb::frame_password::on_back()
{
  close_window();
  return true;
} // frame_password::on_back()

/*----------------------------------------------------------------------------*/
/**
 * \brief Execute a command entered by the user.
 * \param command The tokens read on the command line.
 */
void ptb::frame_password::execute_command
( const std::vector<std::string>& command ) const
{
  CLAW_PRECOND( !command.empty() );

  if ( command[0] == "goto" )
    command_load_level(command);
  else
    claw::logger << claw::log_warning << "Unknow command '" << command[0]
                 << "'" << claw::lendl;
} // frame_password::execute_command()

/*----------------------------------------------------------------------------*/
/**
 * \brief Execute a command entered by the user.
 * \param command The tokens read on the command line.
 */
void ptb::frame_password::command_load_level
( const std::vector<std::string>& command ) const
{
  CLAW_PRECOND( command[0] == "goto" );

  if ( command.size() == 2 )
    {
      bear::engine::game& game = bear::engine::game::get_instance();

      if ( game.level_exists(command[1]) )
        {
          game.one_player_game();
          game.get_game_variables().set<std::string>
            ("scenario/next_level", command[1]);
          game.set_waiting_level("loading");
        }
      else
        claw::logger << claw::log_warning << "goto: no level named '"
                 << command[1] << "'." << claw::lendl;
    }
  else
    claw::logger << claw::log_warning << "goto: one parameter required, "
                 << (command.size()-1) << " given." << claw::lendl;
} // frame_password::command_load_level()
