/*
  Plee the Bear

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [PTB] in the subject of your mails.
*/
/**
 * \file player_arrows_layer.cpp
 * \brief Implementation of the ptb::player_arrows_layer class.
 * \author Sebastien Angibaud
 */
#include "ptb/layer/player_arrows_layer.hpp"
#include "ptb/export.hpp"
#include "engine/game.hpp"
#include "engine/camera.hpp"
#include "universe/zone.hpp"
#include "text/font.hpp"

#include <algorithm>
#include <claw/algorithm.hpp>
#include <claw/functional.hpp>
#include <claw/assert.hpp>

GUI_LAYER_EXPORT( player_arrows_layer, ptb )

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param p A pointer to the player.
 * \param arrow The arrow to display.
 * \param f The font to use to display the distance.
 */
ptb::player_arrows_layer::player_data::player_data
( const bear::engine::pointer_to_player& p, const bear::visual::sprite& arrow,
  const bear::text::font& f )
  : m_arrow(arrow), m_distance(NULL, f), m_player(p), m_visible(false)
{
  m_distance.set_auto_size(true);
  m_distance.set_text("0");
  m_distance.set_position( 0,0 );
} // player_arrows_layer::player_data::player_data()

/*----------------------------------------------------------------------------*/
/**
 * \brief Render the arrow.
 * \param screen The screen on which we will draw.
 */
void ptb::player_arrows_layer::player_data::render
( bear::visual::screen& screen ) const
{
  if (m_visible)
    {
      screen.render
        ( m_position.cast_value_type_to<int>(), m_arrow, m_angle );
      m_distance.render( screen );
    }
} // player_arrows_layer::player_data::render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Set the position and the angle of the arrow.
 * \param visible_area The visible part of the level.
 */
void ptb::player_arrows_layer::player_data::adjust_arrow
( const bear::universe::rectangle_type& visible_area )
{
  m_visible = false;

  if (m_player)
    {    
      const claw::math::rectangle<bear::universe::coordinate_type> box =
        m_player->get_bounding_box();

      bear::universe::zone::position zone_position =
        bear::universe::zone::find( box, visible_area );

      if ( zone_position != bear::universe::zone::middle_zone )
        {
          adjust_position( visible_area );
          adjust_angle( visible_area );
          adjust_distance( visible_area );

          m_visible = true;
        }
    }
} // player_arrows_layer::player_data::adjust_arrow()

/*----------------------------------------------------------------------------*/
/**
 * \brief Adjust the position of the arrow.
 * \param visible_area The visible part of the layer.
 */
void ptb::player_arrows_layer::player_data::adjust_position
( const bear::universe::rectangle_type& visible_area )
{
  bear::universe::position_type result;
  
  bear::universe::position_type pos_player( m_player->get_center_of_mass() );
  bear::universe::position_type center(visible_area.position);
  center.x += visible_area.width/2;
  center.y += visible_area.height/2;
  
  claw::math::vector_2d<double> vect(center, m_player->get_center_of_mass());
  vect.normalize();
  vect.x *= ( visible_area.width / 2 ) - 50;
  vect.y *= ( visible_area.height / 2 ) - 50;

  m_position = center + vect - visible_area.position - m_arrow.get_size() / 2;
} // player_arrows_layer::player_data::adjust_position()

/*----------------------------------------------------------------------------*/
/**
 * \brief Adjust the angle of the arrow.
 * \param visible_area The visible part of the layer.
 */
void ptb::player_arrows_layer::player_data::adjust_angle
( const bear::universe::rectangle_type& visible_area )
{
  const bear::universe::position_type position_arrow =
    m_position + visible_area.position + m_arrow.get_size() / 2;
  bear::universe::speed_type v =
    m_player->get_center_of_mass() - position_arrow;

  v.normalize();
  
  if ( v.y <= 0 )
    m_angle = -acos(v.x);
  else
    m_angle = acos(v.x);
} // player_arrows_layer::player_data::adjust_angle()

/*----------------------------------------------------------------------------*/
/**
 * \brief Adjust the distance between the player and the arrow.
 * \param visible_area The visible part of the layer.
 */
void ptb::player_arrows_layer::player_data::adjust_distance
( const bear::universe::rectangle_type& visible_area )
{
  bear::universe::position_type relative_position;
  relative_position = m_position + visible_area.position;

  bear::universe::speed_type vect
    (m_player->get_center_of_mass(), relative_position);
  unsigned int dist = (unsigned int)vect.length() / 100;

  std::ostringstream oss;
  oss << dist;
  m_distance.set_text(oss.str());

  bear::universe::position_type text_position(m_position);

  bear::universe::position_type gap;
  gap.x = cos(m_angle) * 50;
  gap.y = - sin(m_angle) * 50;

  text_position.x += m_arrow.width() / 2 - gap.x - m_distance.width() / 2;
  text_position.y += m_arrow.height() / 2 + gap.y - m_distance.height() / 2;

  m_distance.set_position( text_position.cast_value_type_to<unsigned int>() );
} // player_arrows_layer::player_data::adjust_distance()




/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 */
ptb::player_arrows_layer::player_arrows_layer()
  : m_data_1(NULL), m_data_2(NULL)
{

} // player_arrows_layer::player_arrows_layer()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
ptb::player_arrows_layer::~player_arrows_layer()
{
  delete m_data_1;
  delete m_data_2;
} // player_arrows_layer::~player_arrows_layer()

/*----------------------------------------------------------------------------*/
/**
 * \brief Initialise the layer.
 */
void ptb::player_arrows_layer::build()
{
  bear::engine::level_globals& glob = get_level_globals();

  bear::text::font f = glob.get_font("font/fixed_yellow-10x20.fnt");
  const bear::visual::image& image_resource = glob.get_image("gfx/status.tga");

  claw::math::rectangle<unsigned int> box( 101, 0, 47, 27 );
  m_data_1 = new player_data
    ( bear::engine::player::player_name(1),
      bear::visual::sprite(image_resource, box), f );

  box.set( 101, 27, 47, 27 );
  m_data_2 = new player_data
    ( bear::engine::player::player_name(2),
      bear::visual::sprite(image_resource, box), f );
} // player_arrows_layer::build()

/*----------------------------------------------------------------------------*/
/**
 * \brief Do one step in the progression of the layer.
 * \param elapsed_time Elapsed time since the last call.
 */
void
ptb::player_arrows_layer::progress( bear::universe::time_type elapsed_time )
{
  bear::engine::camera::msg_get_focus msg;

  bear::engine::level_globals& glob = get_level_globals();

  glob.send_message( bear::engine::camera::default_name(), msg );

  m_data_1->adjust_arrow(msg.focus);
  m_data_2->adjust_arrow(msg.focus);
} // player_arrows_layer::progress()

/*----------------------------------------------------------------------------*/
/**
 * \brief Render the layer.
 * \param screen The screen on which we'll draw.
 */
void ptb::player_arrows_layer::render( bear::visual::screen& screen ) const
{
  m_data_1->render(screen);
  m_data_2->render(screen);
} // player_arrows_layer::render()
