/*
  Plee the Bear

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [PTB] in the subject of your mails.
*/
/**
 * \file status_layer.cpp
 * \brief Implementation of the ptb::status_layer class.
 * \author Julien Jorge
 */
#include "ptb/layer/status_layer.hpp"

#include "engine/game.hpp"
#include "generic_items/timer_item.hpp"
#include "ptb/item/plee/plee.hpp"
#include "ptb/export.hpp"

GUI_LAYER_EXPORT( status_layer, ptb )

const unsigned int ptb::status_layer::s_width_energy = 100;

/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor.
 * \param e The sprite to use for the energy bar.
 * \param f The font to use to display the texts.
 * \param p The player from which we take the data.
 */
ptb::status_layer::player_status::player_status
( const bear::visual::sprite& e, const bear::text::font& f,
  const bear::engine::pointer_to_player& p )
  : energy(e), stones(NULL, f), player(p)
{
  stones.set_auto_size(true);
  stones.set_text("0");
  stones.set_position( 0, 0 );
} // status_layer::player_status::player_status()

/*----------------------------------------------------------------------------*/
/**
 * \brief Update the values.
 */
void ptb::status_layer::player_status::update()
{
  if (player)
    {
      plee* p = dynamic_cast<plee*>(&(*player));

      if (p != NULL)
        {
          std::ostringstream oss;
          oss << p->get_stones_count();

          stones.set_text(oss.str());
          
          energy.set_width
            ( s_width_energy * p->get_energy() / plee::s_max_energy );
        }
    }
} // status_layer::player_status::update()




/*----------------------------------------------------------------------------*/
/**
 * \brief Constructor
 */
ptb::status_layer::status_layer()
  : m_data_1(NULL), m_data_2(NULL), m_text_time(NULL)
{

} // status_layer::status_layer()

/*----------------------------------------------------------------------------*/
/**
 * \brief Destructor.
 */
ptb::status_layer::~status_layer()
{
  delete m_text_time;
  delete m_data_1;
  delete m_data_2;
} // status_layer::~status_layer()

/*----------------------------------------------------------------------------*/
/**
 * \brief Initialize the layer.
 */
void ptb::status_layer::build()
{
  bear::engine::level_globals& glob = get_level_globals();

  m_font_1 = glob.get_font("font/fixed_yellow-14x28.fnt");
  m_font_2 = glob.get_font("font/fixed_red-14x28.fnt");

  m_text_time = new bear::gui::static_text( NULL, m_font_1 );

  m_text_time->set_auto_size(true);
  m_text_time->set_text("00:00");
  m_text_time->set_position( (get_size().x - m_text_time->width()) / 2 ,10 );
  
  const bear::visual::image& image_resource = glob.get_image("gfx/status.tga");

  m_sprite_no_energy =
    bear::visual::sprite
    ( image_resource, claw::math::rectangle<unsigned int>( 0, 0, 100, 20 ) );

  m_sprite_stone =
    bear::visual::sprite
    ( image_resource, claw::math::rectangle<unsigned int>( 0, 40, 24, 24 ) );

  bear::visual::sprite energy_spr
    ( image_resource, claw::math::rectangle<unsigned int>( 0, 20, 100, 20 ) );

  m_data_1 = new player_status
    ( energy_spr, m_font_1, bear::engine::player::player_name(1) );
  m_data_2 = new player_status
    ( energy_spr, m_font_1, bear::engine::player::player_name(2) );
} // status_layer::build()

/*----------------------------------------------------------------------------*/
/**
 * \brief Get the players status and update local values.
 */
void ptb::status_layer::progress( bear::universe::time_type elapsed_time )
{
  progress_time(elapsed_time);

  m_data_1->update();
  m_data_2->update();

  m_data_1->stones.set_position(40, 10);
  m_data_2->stones.set_position
    ( get_size().x - 40 - m_data_2->stones.get_size().x, 10 );
} // status_layer::progress()

/*----------------------------------------------------------------------------*/
/**
 * \brief Update the time text.
 */
void ptb::status_layer::progress_time( bear::universe::time_type elapsed_time )
{
  bear::timer_item::msg_get_value msg;
  bear::engine::level_globals& glob = get_level_globals();

  if ( glob.send_message( "level_timer", msg ) )
    {
      bear::universe::time_type time = msg.value;
      unsigned int sec = (unsigned int)time % 60;
      unsigned int min = (unsigned int)time / 60;
  
      std::ostringstream oss_time;
      if ( min < 10 )
        oss_time << " ";
      oss_time << min;
  
      oss_time << ":";
    
      if ( sec < 10 ) 
        oss_time << "0";
      oss_time << sec;
  
      if ( ( min == 0 ) && ( sec <= 30 ) && 
	   ( (unsigned int)((time - 60*min - sec )*4)%2 == 1) ) 
	{
	  m_text_time->set_font(m_font_2);
	  m_text_time->set_text(oss_time.str());
	}
      else
	{   
	  m_text_time->set_font(m_font_1);
	  m_text_time->set_text(oss_time.str());
	}
    }
} // status_layer::progress_time()

/*----------------------------------------------------------------------------*/
/**
 * \brief Render the players status.
 * \param screen The screen on which we will render.
 */
void ptb::status_layer::render( bear::visual::screen& screen ) const
{
  // render first player status
  if (m_data_1->player)
    render_player1( screen );

  // render second player status (if any)
  if (m_data_2->player)
    render_player2( screen );

  m_text_time->render( screen );
} // status_layer::render()

/*----------------------------------------------------------------------------*/
/**
 * \brief Draw the status of the first player.
 * \param screen The screen on which we'll draw.
 */
void ptb::status_layer::render_player1( bear::visual::screen& screen ) const
{
  m_data_1->stones.render( screen );

  claw::math::coordinate_2d<unsigned int> pos(10, 12);
  screen.render( pos, m_sprite_stone );
  pos.y = 45;
  screen.render( pos, m_sprite_no_energy );

  screen.render( pos, m_data_1->energy );
} // status_layer::render_player1()

/*----------------------------------------------------------------------------*/
/**
 * \brief Draw the status of the second player.
 * \param screen The screen on which we'll draw.
 */
void ptb::status_layer::render_player2( bear::visual::screen& screen ) const
{
  m_data_2->stones.render( screen );

  claw::math::coordinate_2d<unsigned int> pos(get_size().x - 35, 12);
  screen.render( pos, m_sprite_stone );
  pos.x = get_size().x - 110;
  pos.y = 45;
  screen.render( pos, m_sprite_no_energy );

  screen.render( pos, m_data_2->energy );
} // status_layer::render_player2()
